<?php

class Api_Crm_Leadtags extends Api_Leads_Leadtags
{


    /**
     * @var array
     */
    public static $_errorCodes = array(
        '1' => 'Access denied',
        '2' => 'Tag name is missing',
        '3' => 'Tag label is missing',
        '4' => 'Tag with such name already exists',
        '5' => 'Tag with such id not found',
        '6' => 'Tag id missing',
        '7' => 'Tag not found'
    );


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('tag_name', 'label');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_GUEST => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $translator = Zend_Registry::get('Zend_Translate');
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
        $authHeader = $this->_request->getHeader('Authorization');
        if (empty($authHeader)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }
        $token = Tools_LeadExternalApiTools::verify($authHeader);
        if ($token === false) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[1]), self::REST_STATUS_UNAUTHORIZED);
        }

    }


    /**
     * Get lead tags by id
     *
     * Resource:
     * : /api/crm/leadtags/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'plt.tag_name'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $searchTerm = filter_var($this->_request->getParam('searchTerm'), FILTER_SANITIZE_STRING);
        $searchByTagName = filter_var($this->_request->getParam('searchByTagName'), FILTER_SANITIZE_STRING);

        Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
            $this->_request->getParams(), Tools_LeadExternalApiTools::LEAD_TAGS_ENDPOINT, 'GET');

        $leadTagsDataMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
        if ($id) {
            $where = $leadTagsDataMapper->getDbTable()->getAdapter()->quoteInto('plt.id = ?', $id);
            $data = $leadTagsDataMapper->fetchAll($where);
        } elseif (!empty($searchTerm)) {
            $where = $leadTagsDataMapper->getDbTable()->getAdapter()->quoteInto('plt.tag_name LIKE ?',
                $searchTerm . '%');
            $data = $leadTagsDataMapper->fetchAll($where, $sortOrder, $limit, $offset, true);
        } elseif (!empty($searchByTagName)) {
            $where = $leadTagsDataMapper->getDbTable()->getAdapter()->quoteInto('plt.tag_name = ?', $searchByTagName);
            $data = $leadTagsDataMapper->fetchAll($where, $sortOrder, $limit, $offset, true, true);
        } else {
            $data = $leadTagsDataMapper->fetchAll(null, $sortOrder, $limit, $offset);
        }

        return $data;
    }

    /**
     * Create new lead tag
     *
     * Resource:
     * : /api/crm/leadtags/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if(empty($data)) {
            $data = $this->getRequest()->getParams();
        }
        $translator = Zend_Registry::get('Zend_Translate');

        $data = Tools_LeadTools::camelCaseKeysToUnderscore($data, true);
        $data['tag_name'] = rtrim(preg_replace('~[^\w\s\_]~ui', '', $data['tag_name']));
        $data['label'] = preg_replace('~[^\w\s\_]~ui', '', $data['label']);

        if (empty($data['tag_name'])) {
            return array(
                'status' => false,
                'errorCode' => '2',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[2])
            );
        }

        if (empty($data['label'])) {
            return array(
                'status' => false,
                'errorCode' => '3',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[3])
            );
        }

        $leadsTagMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
        $leadsTagModel = new Leads_Model_LeadsTagModel();
        $leadsTagModel->setOptions($data);

        $validator = new Zend_Validate_Db_NoRecordExists(array(
            'table' => 'plugin_leads_tags',
            'field' => 'tag_name'
        ));

        if (!$validator->isValid($data['tag_name'])) {
            return array(
                'status' => false,
                'errorCode' => '4',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[4])
            );
        }

        $leadsTagModel = $leadsTagMapper->save($leadsTagModel);

        Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
            $this->_request->getParams(), Tools_LeadExternalApiTools::LEAD_TAGS_ENDPOINT, 'POST');

        return array(
            'status' => true,
            'tagInfo' => $leadsTagModel->toArray(),
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Tag has been created')
        );
    }

    /**
     * Update lead tag data
     *
     * Resource:
     * : /api/crm/leadtags/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     * : lead tag id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if(empty($data)) {
            $data = $this->getRequest()->getParams();
        }
        if (!empty($data['id'])) {
            $data = Tools_LeadTools::camelCaseKeysToUnderscore($data, true);
            $translator = Zend_Registry::get('Zend_Translate');
            $leadTagId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadTagsMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
            $leadTagsModel = $leadTagsMapper->find($leadTagId);
            if (!$leadTagsModel instanceof Leads_Model_LeadsTagModel) {
                return array(
                    'status' => false,
                    'errorCode' => '5',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[5])
                );
            }

            $data['tag_name'] = rtrim(preg_replace('~[^\w\s\_]~ui', '', $data['tag_name']));
            $data['label'] = preg_replace('~[^\w\s\_]~ui', '', $data['label']);
            if (empty($data['tag_name'])) {
                return array(
                    'status' => false,
                    'errorCode' => '2',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[2])
                );
            }

            if (empty($data['label'])) {
                return array(
                    'status' => false,
                    'errorCode' => '3',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[3])
                );
            }
            $oldTag = $leadTagsModel->getTagName();
            $currentTag = $data['tag_name'];
            if ($oldTag !== $currentTag) {
                $validateTagExists = new Zend_Validate_Db_RecordExists(array(
                    'table' => 'plugin_leads_tags',
                    'field' => 'tag_name'
                ));
                if ($validateTagExists->isValid($currentTag)) {
                    return array(
                        'status' => false,
                        'errorCode' => '4',
                        'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[4])
                    );
                }
            }

            $leadTagsModel->setOptions($data);
            $leadTagsModel = $leadTagsMapper->save($leadTagsModel);

            Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
                $this->_request->getRawBody(), Tools_LeadExternalApiTools::LEAD_TAGS_ENDPOINT, 'PUT');

            return array(
                'status' => true,
                'tagInfo' => $leadTagsModel->toArray(),
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Tag has been updated')
            );
        }

    }

    /**
     * Delete lead tag
     *
     * Resource:
     * : /api/crm/leadtags/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     * : lead tag id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $translator = Zend_Registry::get('Zend_Translate');
        if (!$id) {
            return array(
                'status' => false,
                'errorCode' => '6',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[6])
            );
        }
        $translator = Zend_Registry::get('Zend_Translate');

        $leadsTagsMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
        $leadTagsModel = $leadsTagsMapper->find($id);
        if ($leadTagsModel instanceof Leads_Model_LeadsTagModel) {
            $leadsTagsMapper->delete($id);
            Tools_LeadExternalApiTools::logRequest($this->_request->getHeader('Authorization'),
                $this->_request->getParams(), Tools_LeadExternalApiTools::LEAD_TAGS_ENDPOINT, 'DELETE');
            return array('status' => true, 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Tag has been deleted'));
        }

        return array(
            'status' => false,
            'errorCode' => '7',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.self::$_errorCodes[7])
        );

    }

}
