<?php

class Api_Leads_Leadevents extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('event_name', 'event_label');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead events by id
     *
     * Resource:
     * : /api/leads/events/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'ple.event_name'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $searchTerm = filter_var($this->_request->getParam('searchTerm'), FILTER_SANITIZE_STRING);
        $searchByEventName = filter_var($this->_request->getParam('searchByEventName'), FILTER_SANITIZE_STRING);

        $leadEventsDataMapper = Leads_Mapper_LeadsEventsMapper::getInstance();
        if ($id) {
            $where = $leadEventsDataMapper->getDbTable()->getAdapter()->quoteInto('ple.id = ?', $id);
            $data = $leadEventsDataMapper->fetchAll($where);
        }  elseif (!empty($searchTerm)) {
            $where = $leadEventsDataMapper->getDbTable()->getAdapter()->quoteInto('ple.event_name LIKE ?',
                $searchTerm . '%');
            $data = $leadEventsDataMapper->fetchAll($where, $sortOrder, $limit, $offset, true);
        } elseif(!empty($searchByEventName)) {
            $where = $leadEventsDataMapper->getDbTable()->getAdapter()->quoteInto('ple.event_name = ?', $searchByEventName);
            $data = $leadEventsDataMapper->fetchAll($where, $sortOrder, $limit, $offset, true, true);
        } else {
            $data = $leadEventsDataMapper->fetchAll(null, $sortOrder, $limit, $offset);
        }

        return $data;
    }

    /**
     * Create new event event
     *
     * Resource:
     * : /api/leads/leadevents/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $data['event_name'] = rtrim(preg_replace('~[^\w\s\_]~ui', '', $this->_request->getParam('event_name')));
        $data['event_label'] = preg_replace('~[^\w\s\_]~ui', '', $this->_request->getParam('event_label'));
        $data['event_external_names'] = array_filter(array_unique(explode(',', filter_var($this->_request->getParam('event_external_names'), FILTER_SANITIZE_STRING))));

        $leadsEventMapper = Leads_Mapper_LeadsEventsMapper::getInstance();
        $leadsEventModel = new Leads_Model_LeadsEventModel();
        $leadsEventModel->setOptions($data);

        $validator = new Zend_Validate_Db_NoRecordExists(array(
            'table' => 'plugin_leads_events',
            'field' => 'event_name'
        ));

        if (!$validator->isValid($data['event_name'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Event with such name already exists'));
        }

        $leadsEventModel = $leadsEventMapper->save($leadsEventModel);

        $this->_responseHelper->success(array('eventInfo' => $leadsEventModel->toArray(), 'message' =>$translator->translate(Leads::TRANSLATION_PREFIX.'Event has been created')));
    }

    /**
     * Update lead event data
     *
     * Resource:
     * : /api/leads/leadevents/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     * : lead event id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $leadEventId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadEventsMapper = Leads_Mapper_LeadsEventsMapper::getInstance();
            $leadEventsModel = $leadEventsMapper->find($leadEventId);
            if (!$leadEventsModel instanceof Leads_Model_LeadsEventModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Config doesn\'t exists'));
            }

            $data['event_name'] = rtrim(preg_replace('~[^\w\s\_]~ui', '', $data['event_name']));
            $data['event_label'] = preg_replace('~[^\w\s\_]~ui', '', $data['event_label']);
            $data['event_external_names'] = array_filter(array_unique(explode(',', filter_var($data['event_external_names'], FILTER_SANITIZE_STRING))));
            $oldEvent = $leadEventsModel->getEventName();
            $currentEvent = $data['event_name'];
            if ($oldEvent !== $currentEvent) {
                $validateEventExists = new Zend_Validate_Db_RecordExists(array(
                    'table' => 'plugin_leads_events',
                    'field' => 'event_name'
                ));
                if ($validateEventExists->isValid($currentEvent)) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You have another event with such name'));
                }
            }

            $leadEventsModel->setOptions($data);
            $leadEventsMapper->deleteEventExternals($data['id']);
            $leadEventsMapper->save($leadEventsModel);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Event has been updated'));
        }

    }

    /**
     * Delete lead event
     *
     * Resource:
     * : /api/leads/leadevents/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     * : lead event id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsEventsMapper = Leads_Mapper_LeadsEventsMapper::getInstance();
        $leadEventsModel = $leadsEventsMapper->find($id);
        if ($leadEventsModel instanceof Leads_Model_LeadsEventModel) {
            $leadsEventsMapper->delete($id);
        } else {
            $this->_error();
        }

    }

}
