<?php

class Api_Leads_Leadimap extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('service_name', 'owner_id', 'host', 'password', 'login');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead imap config by id
     *
     * Resource:
     * : /api/leads/imap/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'plic.service_name'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $searchTerm = filter_var($this->_request->getParam('searchTerm'), FILTER_SANITIZE_STRING);

        $sessionHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $currentUser = $sessionHelper->getCurrentUser();
        $userRoleId = $currentUser->getRoleId();
        $userId = $currentUser->getId();

        $salesOnly = filter_var($this->_request->getParam('salesOnly'), FILTER_SANITIZE_NUMBER_INT);

        $leadImapDataMapper = Leads_Mapper_LeadsImapConfigMapper::getInstance();
        if ($id) {
            $where = $leadImapDataMapper->getDbTable()->getAdapter()->quoteInto('plic.id = ?', $id);
            $data = $leadImapDataMapper->fetchAllData($where);
        }  elseif (!empty($searchTerm)) {
            $where = $leadImapDataMapper->getDbTable()->getAdapter()->quoteInto('plic.service_name LIKE ?',
                $searchTerm . '%');
            $data = $leadImapDataMapper->fetchAllData($where, $sortOrder, $limit, $offset, true);
        } else {
            $where = null;
            if(!empty($salesOnly) && $userRoleId == Shopping::ROLE_SALESPERSON) {
                $where = $leadImapDataMapper->getDbTable()->getAdapter()->quoteInto('plic.owner_id = ?', $userId);
            }

            $data = $leadImapDataMapper->fetchAllData($where, $sortOrder, $limit, $offset);
        }

        if (empty($data)) {
            $data = array();
        } else {
            $data['leadOwners'] = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList();

            if(!empty($salesOnly) && $userRoleId == Shopping::ROLE_SALESPERSON) {
                $data['leadOwners'] = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList(0, array($userId));
            }

            $data['emailsPath'] = Zend_Registry::get('website')['path'].Tools_LeadInboundEmailTools::EMAIL_PROCESS_EMAILS_PATH;
            $data['historicalEmailsPath'] = Zend_Registry::get('website')['path'].Tools_LeadInboundEmailTools::EMAIL_PROCESS_HISTORICAL_EMAILS_PATH;
        }

        return $data;
    }

    /**
     * Create new imap config
     *
     * Resource:
     * : /api/leads/imap/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'),
                'errorCode' => '1'
            ), 200);
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        if (empty($data['process_all_emails'])) {
            $data['process_all_emails'] = '0';
        }

        $connection = Tools_LeadInboundEmailTools::getImapConnection($data['host'], $data['encryption'], $data['login'], $data['password']);
        if ($connection === false) {
            $error = Leads::TRANSLATION_PREFIX.'Can\'t establish connection';
            $imapError = Tools_LeadInboundEmailTools::getImapError();
            if (!empty($imapError)) {
                $error = $imapError;
            }

            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate($error),
                'errorCode' => '1'
            ), 200);
        }

        $leadImapDataMapper = Leads_Mapper_LeadsImapConfigMapper::getInstance();
        $leadImapDataModel = new Leads_Model_LeadsImapConfigModel();
        $leadImapDataModel->setOptions($data);

        $leadImapDataModel = $leadImapDataMapper->save($leadImapDataModel);

        return array(
            'error' => '0',
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'We\'ve just tested your credentials. Everything looks good. Inbound emails will start showing in your leads timeline shortly.'),
            'imapInfo' => $leadImapDataModel->toArray()
        );
    }

    /**
     * Update lead imap data
     *
     * Resource:
     * : /api/leads/imap/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     * : lead imap id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        $translator = Zend_Registry::get('Zend_Translate');

        $tokenValid = Tools_System_Tools::validateToken($data['secureToken'],Leads::LEADS_SECURE_TOKEN);
        if ($tokenValid !== true) {
            $this->_error(array('status' => 'error', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Access denied'), 'errorCode' => '1'), 200);
        }

        if (!empty($data['id'])) {
            if (!isset($data['password']) || empty($data['password'])) {
                $leadImapDataMapper = Leads_Mapper_LeadsImapConfigMapper::getInstance();
                $where = $leadImapDataMapper->getDbTable()->getAdapter()->quoteInto('id = ?', $data['id']);
                $leadImapInfo = $leadImapDataMapper->fetchAllData($where, null, null, null, true, true);
                if (!empty($leadImapInfo)) {
                    $data['password'] = $leadImapInfo['password'];
                }
            }
            
            if (empty($data['process_all_emails'])) {
                $data['process_all_emails'] = '0';
            }

            if(empty($data['fastSave'])) {
                $connection = Tools_LeadInboundEmailTools::getImapConnection($data['host'], $data['encryption'], $data['login'], $data['password']);
                if ($connection === false) {
                    $error = Leads::TRANSLATION_PREFIX.'Can\'t establish connection';
                    $imapError = Tools_LeadInboundEmailTools::getImapError();
                    if (!empty($imapError)) {
                        $error = $imapError;
                    }

                    $this->_error(array(
                        'status' => 'error',
                        'message' => $translator->translate($error),
                        'errorCode' => '1'
                    ), 200);
                }
            }
            $leadImapId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadImapDataMapper = Leads_Mapper_LeadsImapConfigMapper::getInstance();
            $leadImapModel = $leadImapDataMapper->find($leadImapId);
            if (!$leadImapModel instanceof Leads_Model_LeadsImapConfigModel) {
                $this->_error(array(
                    'status' => 'error',
                    'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Config doesn\'t exists'),
                    'errorCode' => '1'
                ), 200);
            }


            $leadImapModel->setOptions($data);
            $leadImapDataMapper->save($leadImapModel);

            $message = $translator->translate(Leads::TRANSLATION_PREFIX.'Updated');

            if(empty($data['fastSave'])) {
                $message = $translator->translate(Leads::TRANSLATION_PREFIX.'We\'ve just tested your credentials. Everything looks good. Inbound emails will start showing in your leads timeline shortly.');
            }

            return array(
                'error' => '0',
                'message' => $message,
                'imapInfo' => $leadImapModel->toArray()
            );
        }

    }

    /**
     * Delete lead imap config
     *
     * Resource:
     * : /api/leads/imap/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     * : lead event id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Access denied'),
                'errorCode' => '1'
            ), 200);
        }
        $leadsImapMapper = Leads_Mapper_LeadsImapConfigMapper::getInstance();
        $leadImapModel = $leadsImapMapper->find($id);
        if ($leadImapModel instanceof Leads_Model_LeadsImapConfigModel) {
            $leadsImapMapper->delete($id);

            return array('error' => '0', 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Record has been deleted'));
        } else {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Record not found'),
                'errorCode' => '1'
            ), 200);
        }

    }

}
