<?php

class Api_Leads_Leadindustry extends Api_Service_Abstract
{

    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('type', 'label');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead by id
     *
     * Resource:
     * : /api/leads/leadindustry/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'pli.type'), FILTER_SANITIZE_STRING);
        $searchTerm = filter_var($this->_request->getParam('searchTerm', null), FILTER_SANITIZE_STRING);
        $leadIndustryMapper = Leads_Mapper_LeadsIndustryMapper::getInstance();
        $data = $leadIndustryMapper->fetchAllData(null, $sortOrder, $limit, $offset, false, false, $searchTerm);
        return $data;
    }

    /**
     * New lead creation
     *
     * Resource:
     * : /api/leads/leadindustry/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        /**
         * Create industry
         */

        $type = trim(filter_var($data['type'], FILTER_SANITIZE_STRING));
        $label = trim(filter_var($data['label'], FILTER_SANITIZE_STRING));

        $validateTypeExists = new Zend_Validate_Db_RecordExists(array(
            'table' => 'plugin_leads_industry',
            'field' => 'type'
        ));
        if ($validateTypeExists->isValid($type)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You have another industry type with such name'));
        }

        $leadsIndustryMapper = Leads_Mapper_LeadsIndustryMapper::getInstance();
        $leadsIndustryModel = new Leads_Model_LeadsIndustryModel();
        $leadsIndustryModel->setType($type);
        $leadsIndustryModel->setLabel($label);
        $leadsIndustryMapper->save($leadsIndustryModel);
        $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Industry has been created'));
    }

    /**
     * Update lead type data
     *
     * Resource:
     * : /api/leads/leadindustry/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead type id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $id= filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadsIndustryMapper = Leads_Mapper_LeadsIndustryMapper::getInstance();
            $leadsIndustryModel = $leadsIndustryMapper->find($id);
            if (!$leadsIndustryModel instanceof Leads_Model_LeadsIndustryModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Config doesn\'t exists'));
            }

            $oldType = $leadsIndustryModel->getType();
            $currentType = $data['type'];
            if ($oldType !== $currentType) {
                $validateTypeExists = new Zend_Validate_Db_RecordExists(array(
                    'table' => 'plugin_leads_industry',
                    'field' => 'type'
                ));
                if ($validateTypeExists->isValid($currentType)) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You have another industry type with such name'));
                }
            }

            $leadsIndustryModel->setOptions($data);
            $leadsIndustryMapper->save($leadsIndustryModel);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Industry has been updated'));
        }

    }

    /**
     * Delete lead
     *
     * Resource:
     * : /api/leads/leadindustry/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : leads id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $translator = Zend_Registry::get('Zend_Translate');
        $leadsIndustryMapper = Leads_Mapper_LeadsIndustryMapper::getInstance();
        $leadIndustryModel =  $leadsIndustryMapper->find($id);
        if ($leadIndustryModel instanceof Leads_Model_LeadsIndustryModel) {
            $leadsIndustryMapper->delete($id);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Industry has been deleted'));
        } else {
            $this->_error();
        }
        $this->_responseHelper->fail();

    }

}
