<?php

class Api_Leads_Leadnotes extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('note');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead notes by note id and lead id
     *
     * Resource:
     * : /api/leads/leadnotes/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'pln.date_created DESC'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $leadId = filter_var($this->_request->getParam('lead_id', false), FILTER_SANITIZE_NUMBER_INT);

        $leadNotesDataMapper = Leads_Mapper_LeadsNoteMapper::getInstance();

        if (!empty($leadId)) {
            $whereLeadId = $leadNotesDataMapper->getDbTable()->getAdapter()->quoteInto('pln.lead_id = ?',
                    $leadId);
        }

        if ($id) {
            $where = $leadNotesDataMapper->getDbTable()->getAdapter()->quoteInto('pln.id = ?', $id);
            if (!empty($whereLeadId)) {
                $where .= ' AND  ' . $whereLeadId;
            }
            $data = $leadNotesDataMapper->fetchAllData($where);
        } else {
            $where = null;
            if (!empty($whereLeadId)) {
                $where = $whereLeadId;
            }
            $data = $leadNotesDataMapper->fetchAllData($where, $sortOrder, $limit, $offset, true);
        }

        return $data;
    }

    /**
     * Create new lead notes
     *
     * Resource:
     * : /api/leads/leadnotes/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'The note text is missing'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $leadsNotesMapper = Leads_Mapper_LeadsNoteMapper::getInstance();
        $leadsNoteModel = new Leads_Model_LeadsNoteModel();
        $leadsNoteModel->setOptions($data);
        $leadsNoteModel->setDateCreated(Tools_System_Tools::convertDateFromTimezone('now'));
        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $userId = $session->getCurrentUser()->getId();
        $leadsNoteModel->setUserId($userId);

        $leadsNotesMapper->save($leadsNoteModel);

        $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Note has been added'));
    }

    /**
     * Update lead notes data
     *
     * Resource:
     * : /api/leads/leadnotes/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead note id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $leadId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadNoteMapper = Leads_Mapper_LeadsNoteMapper::getInstance();
            $leadNoteModel = $leadNoteMapper->find($leadId);
            if (!$leadNoteModel instanceof Leads_Model_LeadsNoteModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Note doesn\'t exists'));
            }

            $leadNoteModel->setOptions($data);
            $leadNoteMapper->save($leadNoteModel);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Note has been updated'));
        }

    }

    /**
     * Delete lead stage
     *
     * Resource:
     * : /api/leads/leadnotes/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead note id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsNoteMapper = Leads_Mapper_LeadsNoteMapper::getInstance();
        $leadNoteModel = $leadsNoteMapper->find($id);
        if ($leadNoteModel instanceof Leads_Model_LeadsNoteModel) {
            $leadsNoteMapper->delete($id);
        } else {
            $this->_error();
        }

    }

}
