<?php

class Api_Leads_Leadopportunity extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array(
        'Opportunity name'     => 'opportunity_name',
        'Lead id'              => 'lead_id',
        'Opportunity stage id' => 'opportunity_stage_id',
        'Opportunity value'    => 'opportunity_value',
        'Expected closed date' => 'expected_close_date'
    );


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead opportunity by id
     *
     * Resource:
     * : /api/leads/leadopportunity/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $leadId = filter_var($this->_request->getParam('lead_id'), FILTER_SANITIZE_NUMBER_INT);
        $excludeEmptyOpportunityStages = filter_var($this->_request->getParam('excludeEmptyOpportunityStages'), FILTER_SANITIZE_STRING);

        $leadOpportunityDataMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
        if ($id) {
            $where = $leadOpportunityDataMapper->getDbTable()->getAdapter()->quoteInto('plo.id = ?', $id);
            $data = $leadOpportunityDataMapper->fetchAll($where);
        } else {
            $where = null;
            if (!empty($leadId)) {
                $where = $leadOpportunityDataMapper->getDbTable()->getAdapter()->quoteInto('plo.lead_id = ?', $leadId);
            }
            $data = $leadOpportunityDataMapper->fetchAll($where);
            $leadOpportunityDataMapper = Leads_Mapper_LeadsOpportunityStageMapper::getInstance();
            if (!empty($excludeEmptyOpportunityStages)) {
                $excludeEmptyOpportunityStages = true;
            }
            $leadOpportunityTypeMapper = Leads_Mapper_LeadsOpportunityTypeMapper::getInstance();
            $data['opportunityStages'] = $leadOpportunityDataMapper->getOpportunityStages($excludeEmptyOpportunityStages);
            $data['defaultStage'] = $leadOpportunityDataMapper->getDefaultStage();
            $data['opportunityTypes'] = $leadOpportunityTypeMapper->getOpportunityTypesPairs();
        }

        return $data;
    }


    public function postAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (empty($data)) {
            $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        }
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return array_search($param, $this->_mandatoryParams);
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_responseHelper->fail(array('missingParams' => $fieldDataMissing));
        }

        if (!preg_match('~^\d+?$~', $data['opportunity_value'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Enter valid opportunity value. Ex: 100'));
        }

        $secureToken = filter_var($data[Tools_System_Tools::CSRF_SECURE_TOKEN], FILTER_SANITIZE_STRING);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        if (!empty($data['expected_close_date'])) {
            $data['expected_close_date'] = Tools_System_Tools::convertDateFromTimezone($data['expected_close_date']);
        }

        if (!empty($data['duplicate_quote_id'])) {
            $quoteMapper = Quote_Models_Mapper_QuoteMapper::getInstance();
            $quoteModel = $quoteMapper->find($data['duplicate_quote_id']);
            if (!$quoteModel instanceof Quote_Models_Model_Quote) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Quote for duplication not found'));
            }
        }

        $leadOpportunityMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
        $leadOpportunityModel = $leadOpportunityMapper->getOpportunityByOpportunityNameStageIdLeadId($data['opportunity_name'], $data['opportunity_stage_id'], $data['lead_id']);
        if ($leadOpportunityModel instanceof Leads_Model_LeadsOpportunityModel) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity with such name and stage already exists'));
        }

        $leadOpportunityModel = new Leads_Model_LeadsOpportunityModel();

        $leadOpportunityModel->setOptions($data);


        $session = Zend_Controller_Action_HelperBroker::getStaticHelper('session');
        $creatorId = $session->getCurrentUser()->getId();

        $leadOpportunityModel->setCreatorId($creatorId);
        $leadOpportunityModel->setCreatedAt(Tools_System_Tools::convertDateFromTimezone('now'));

        $leadOpportunityModel = $leadOpportunityMapper->save($leadOpportunityModel);

        $this->_responseHelper->success(array(
            'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity has been created'),
            'opportunityId' => $leadOpportunityModel->getId()
        ));
    }


    /**
     * Update lead opportunity data
     *
     * Resource:
     * : /api/leads/leadopportunity/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead opportunity id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            if (!preg_match('~^\d+?$~', $data['opportunity_value'])) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Enter valid opportunity value. Ex: 100'));
            }

            if (!empty($data['expected_close_date'])) {
                $data['expected_close_date'] = Tools_System_Tools::convertDateFromTimezone($data['expected_close_date']);
            } else {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
            }

            $id = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);

            $leadOpportunityMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
            $leadOpportunityModel = $leadOpportunityMapper->find($id);
            if (!$leadOpportunityModel instanceof Leads_Model_LeadsOpportunityModel) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity doesn\'t exist'));
            }

            if (!empty($data['duplicate_quote_id'])) {
                $quoteMapper = Quote_Models_Mapper_QuoteMapper::getInstance();
                $quoteModel = $quoteMapper->find($data['duplicate_quote_id']);
                if (!$quoteModel instanceof Quote_Models_Model_Quote) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Quote for duplication not found'));
                }
            }


            $previousOpportunityId = $leadOpportunityModel->getOpportunityStageId();

            $leadOpportunityModelExists = $leadOpportunityMapper->getOpportunityByOpportunityNameStageIdLeadId($data['opportunity_name'], $data['opportunity_stage_id'], $data['lead_id'], $leadOpportunityModel->getId());
            if ($leadOpportunityModelExists instanceof Leads_Model_LeadsOpportunityModel) {
                $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity with such name and stage already exists'));
            }

            $leadOpportunityModel->setOptions($data);

            if ($previousOpportunityId !== $data['opportunity_stage_id']) {
                $leadOpportunityModel->setChangeStageDate(Tools_System_Tools::convertDateFromTimezone('now'));
            }

            $leadOpportunityMapper->save($leadOpportunityModel);
            $this->_responseHelper->success(array('message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity has been updated'), 'opportunityId' => $leadOpportunityModel->getId()));
        }

    }

    /**
     * Delete lead opportunity
     *
     * Resource:
     * : /api/leads/leadopportunity/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead opportunity id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (empty($id)) {
            $this->_error();
        }
        $leadsOpportunityMapper = Leads_Mapper_LeadsOpportunityMapper::getInstance();
        $leadOpportunityModel = $leadsOpportunityMapper->find($id);
        if ($leadOpportunityModel instanceof Leads_Model_LeadsOpportunityModel) {
            $quoteMapper = Quote_Models_Mapper_QuoteMapper::getInstance();
            $leadOpportunityQuoteMapper = Leads_Mapper_LeadsOpportunityQuoteMapper::getInstance();
            $opportunityQuotesIds = $leadOpportunityQuoteMapper->getOpportunityQuotesIds($leadOpportunityModel->getId());
            if (!empty($opportunityQuotesIds)) {
                $where = $leadOpportunityQuoteMapper->getDbTable()->getAdapter()->quoteInto('id IN (?)', $opportunityQuotesIds);
                $quotes = $quoteMapper->fetchAll($where);
                if (!empty($quotes)) {
                    foreach ($quotes as $quote) {
                        $quoteMapper->delete($quote);
                    }
                }
            }
            $leadsOpportunityMapper->delete($leadOpportunityModel->getId());
            $translator = Zend_Registry::get('Zend_Translate');
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Opportunity has been deleted'));
        }

        $this->_error();

    }

}
