<?php

class Api_Leads_Leadsocialservices extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('service_name', 'service_label');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead social services by id
     *
     * Resource:
     * : /api/leads/leadsocialservices/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'plcsc.service_name'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);

        $leadsSocialServicesConfigMapper = Leads_Mapper_LeadsSocialServicesConfigMapper::getInstance();
        if ($id) {
            $where = $leadsSocialServicesConfigMapper->getDbTable()->getAdapter()->quoteInto('plcsc.id = ?', $id);
            $data = $leadsSocialServicesConfigMapper->fetchAll($where, null, null, null, true, true);

        } else {
            $data = $leadsSocialServicesConfigMapper->fetchAll(null, $sortOrder, $limit, $offset);
        }

        return $data;
    }

    /**
     * Create new lead social service
     *
     * Resource:
     * : /api/leads/leadsocialservices/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $data['service_name'] = preg_replace('~[^\w_]~ui', '', $data['service_name']);
        $leadsSocialServicesConfigMapper = Leads_Mapper_LeadsSocialServicesConfigMapper::getInstance();
        $leadsSocialServiceModel = new Leads_Model_LeadsSocialServiceConfigModel();
        $leadsSocialServiceModel->setOptions($data);

        $validator = new Zend_Validate_Db_NoRecordExists(array(
            'table' => 'plugin_leads_social_services_config',
            'field' => 'service_name'
        ));

        if (!$validator->isValid($data['service_name'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Tag with such name already exists'));
        }

        $leadsSocialServicesConfigMapper->save($leadsSocialServiceModel);

        $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Service has been created'));
    }

    /**
     * Update lead social service data
     *
     * Resource:
     * : /api/leads/leadsocialservices/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     * : lead social service id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $leadServiceId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadsSocialServicesConfigMapper = Leads_Mapper_LeadsSocialServicesConfigMapper::getInstance();
            $leadsSocialServiceModel = $leadsSocialServicesConfigMapper->find($leadServiceId);
            if (!$leadsSocialServiceModel instanceof Leads_Model_LeadsSocialServiceConfigModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Config doesn\'t exists'));
            }

            $oldServiceName = $leadsSocialServiceModel->getServiceName();

            $data['service_name'] = preg_replace('~[^\w_]~ui', '', $data['service_name']);
            $currentServiceName = $data['service_name'];
            if ($oldServiceName !== $currentServiceName) {
                $validateServiceExists = new Zend_Validate_Db_RecordExists(array(
                    'table' => 'plugin_leads_social_services_config',
                    'field' => 'service_name'
                ));
                if ($validateServiceExists->isValid($currentServiceName)) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You have another service with such name'));
                }
            }

            $leadsSocialServiceModel->setOptions($data);
            $leadsSocialServicesConfigMapper->save($leadsSocialServiceModel);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Service has been updated'));
        }

    }

    /**
     * Delete lead social service
     *
     * Resource:
     * : /api/leads/leadsocialservices/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     * : lead social service id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsSocialServicesConfigMapper = Leads_Mapper_LeadsSocialServicesConfigMapper::getInstance();
        $leadsSocialServiceModel = $leadsSocialServicesConfigMapper->find($id);
        if ($leadsSocialServiceModel instanceof Leads_Model_LeadsSocialServiceConfigModel) {
            $leadsSocialServicesConfigMapper->delete($id);
        } else {
            $this->_error();
        }

    }

}
