<?php

class Api_Leads_Leadsource extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('source_name');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead source by id
     *
     * Resource:
     * : /api/leads/leadsource/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'pls.source_name'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $autocomplete =  filter_var($this->_request->getParam('autocomplete'), FILTER_SANITIZE_STRING);
        $searchTerm = filter_var($this->_request->getParam('searchTerm'), FILTER_SANITIZE_STRING);
        $system = filter_var($this->_request->getParam('system'), FILTER_SANITIZE_STRING);

        $leadSourceDataMapper = Leads_Mapper_LeadsSourceMapper::getInstance();
        if ($id) {
            $where = $leadSourceDataMapper->getDbTable()->getAdapter()->quoteInto('pls.id = ?', $id);
            $data = $leadSourceDataMapper->fetchAllData($where);
        } elseif(!empty($autocomplete)) {
            $where = $leadSourceDataMapper->getDbTable()->getAdapter()->quoteInto('source_name LIKE ?', $searchTerm.'%');
            if (empty($system)) {
                $where .= ' AND '. $leadSourceDataMapper->getDbTable()->getAdapter()->quoteInto('`system` <> ?', '1');
            }
            $data = $leadSourceDataMapper->fetchAllData($where, null, null, null, true);
        } else {
            $data = $leadSourceDataMapper->fetchAllData(null, $sortOrder, $limit, $offset);
        }

        return $data;
    }

    /**
     * Create new lead source
     *
     * Resource:
     * : /api/leads/leadsource/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $leadsSourceMapper = Leads_Mapper_LeadsSourceMapper::getInstance();
        $leadsSourceModel = new Leads_Model_LeadsSourceModel();
        $leadsSourceModel->setOptions($data);

        $validator = new Zend_Validate_Db_NoRecordExists(array(
            'table' => 'plugin_leads_source',
            'field' => 'source_name'
        ));

        if (!$validator->isValid($data['source_name'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead source with such name already exists'));
        }

        $leadsSourceMapper->save($leadsSourceModel);

        $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Lead has been created'));
    }

    /**
     * Update lead source data
     *
     * Resource:
     * : /api/leads/leadsource/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     * : lead source id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $leadSourceId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadSourceMapper = Leads_Mapper_LeadsSourceMapper::getInstance();
            $leadSourceModel = $leadSourceMapper->find($leadSourceId);
            if (!$leadSourceModel instanceof Leads_Model_LeadsSourceModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Config doesn\'t exists'));
            }

            $oldSource = $leadSourceModel->getSourceName();
            $currentSource = $data['source_name'];
            if ($oldSource !== $currentSource) {
                $validateSourceExists = new Zend_Validate_Db_RecordExists(array(
                    'table' => 'plugin_leads_source',
                    'field' => 'source_name'
                ));
                if ($validateSourceExists->isValid($currentSource)) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You have another lead source with such name'));
                }
            }

            $leadSourceModel->setOptions($data);
            $leadSourceMapper->save($leadSourceModel);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Lead source has been updated'));
        }

    }

    /**
     * Delete lead source
     *
     * Resource:
     * : /api/leads/leadsource/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     * : lead source id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsSourceMapper = Leads_Mapper_LeadsSourceMapper::getInstance();
        $leadSourceModel = $leadsSourceMapper->find($id);
        if ($leadSourceModel instanceof Leads_Model_LeadsSourceModel) {
            $leadsSourceMapper->delete($id);
        } else {
            $this->_error();
        }

    }

}
