<?php

class Api_Leads_Leadstage extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('stageName');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead stage by id
     *
     * Resource:
     * : /api/leads/leadstage/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'pls.stage_name'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);

        $leadStageDataMapper = Leads_Mapper_LeadsStageMapper::getInstance();
        if ($id) {
            $where = $leadStageDataMapper->getDbTable()->getAdapter()->quoteInto('pls.id = ?', $id);
            $data = $leadStageDataMapper->fetchAllData($where);
        } else {
            $data = $leadStageDataMapper->fetchAllData(null, $sortOrder, $limit, $offset);
        }

        return $data;
    }

    /**
     * Create new lead stage
     *
     * Resource:
     * : /api/leads/leadstage/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $leadsStageMapper = Leads_Mapper_LeadsStageMapper::getInstance();
        $leadsStageModel = new Leads_Model_LeadsStageModel();
        $leadsStageModel->setOptions($data);

        $validator = new Zend_Validate_Db_NoRecordExists(array(
            'table' => 'plugin_leads_stage',
            'field' => 'stage_name'
        ));

        if (!$validator->isValid($data['stageName'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead stage with such name already exists'));
        }

        $data = $leadsStageMapper->save($leadsStageModel);

        $this->_responseHelper->success(array('stageId' => $data->getId(), 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Stage has been created')));
    }

    /**
     * Update lead stage data
     *
     * Resource:
     * : /api/leads/leadstage/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead stage id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $leadId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadStageMapper = Leads_Mapper_LeadsStageMapper::getInstance();
            $leadStageModel = $leadStageMapper->find($leadId);
            if (!$leadStageModel instanceof Leads_Model_LeadsStageModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Config doesn\'t exists'));
            }

            $oldStage = $leadStageModel->getStageName();
            $newStage = $data['stage_name'];
            if ($oldStage !== $newStage) {
                $validateStageExists = new Zend_Validate_Db_RecordExists(array(
                    'table' => 'plugin_leads_stage',
                    'field' => 'stage_name'
                ));
                if ($validateStageExists->isValid($newStage)) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You have another lead stage with such name'));
                }
            }

            if (!empty($data['default']) && $data['default'] === '1') {
                $leadStageMapper->disableDefaultStage();
                $leadStageMapper->assignDefaultStage($leadId);
            }

            $leadStageModel->setOptions($data);
            $leadStageMapper->save($leadStageModel);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Lead stage has been updated'));
        } else {
            $leadStageMapper = Leads_Mapper_LeadsStageMapper::getInstance();
            $leadStageMapper->disableDefaultStage();
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Lead stage has been updated'));
        }

    }

    /**
     * Delete lead stage
     *
     * Resource:
     * : /api/leads/leadstage/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead stage id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsStageMapper = Leads_Mapper_LeadsStageMapper::getInstance();
        $leadStageModel = $leadsStageMapper->find($id);
        if ($leadStageModel instanceof Leads_Model_LeadsStageModel) {
            $defaultStage = $leadsStageMapper->getDefaultStage();
            if($defaultStage instanceof Leads_Model_LeadsStageModel) {
                $defaultStageId = $defaultStage->getId();
                $leadMapper = Leads_Mapper_LeadsMapper::getInstance();
                $leadMapper->assignDefaultStageForLeads($defaultStageId, $id);
            }

            $leadsStageMapper->delete($id);
        } else {
            $this->_error();
        }

    }

}
