<?php

class Api_Leads_Leadtags extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('tag_name', 'label');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead tags by id
     *
     * Resource:
     * : /api/leads/leadtags/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'plt.tag_name'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $searchTerm = filter_var($this->_request->getParam('searchTerm'), FILTER_SANITIZE_STRING);
        $searchByTagName = filter_var($this->_request->getParam('searchByTagName'), FILTER_SANITIZE_STRING);

        $leadTagsDataMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
        if ($id) {
            $where = $leadTagsDataMapper->getDbTable()->getAdapter()->quoteInto('plt.id = ?', $id);
            $data = $leadTagsDataMapper->fetchAll($where);
        }  elseif (!empty($searchTerm)) {
            $where = $leadTagsDataMapper->getDbTable()->getAdapter()->quoteInto('plt.tag_name LIKE ?',
                $searchTerm . '%');
            $data = $leadTagsDataMapper->fetchAll($where, $sortOrder, $limit, $offset, true);
        } elseif(!empty($searchByTagName)) {
            $where = $leadTagsDataMapper->getDbTable()->getAdapter()->quoteInto('plt.tag_name = ?', $searchByTagName);
            $data = $leadTagsDataMapper->fetchAll($where, $sortOrder, $limit, $offset, true, true);
        } else {
            $data = $leadTagsDataMapper->fetchAll(null, $sortOrder, $limit, $offset);
        }

        return $data;
    }

    /**
     * Create new lead tag
     *
     * Resource:
     * : /api/leads/leadtags/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $data['tag_name'] = rtrim(preg_replace('~[^\w\s\_]~ui', '', $this->_request->getParam('tag_name')));
        $data['label'] = preg_replace('~[^\w\s\_]~ui', '', $this->_request->getParam('label'));

        $leadsTagMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
        $leadsTagModel = new Leads_Model_LeadsTagModel();
        $leadsTagModel->setOptions($data);

        $validator = new Zend_Validate_Db_NoRecordExists(array(
            'table' => 'plugin_leads_tags',
            'field' => 'tag_name'
        ));

        if (!$validator->isValid($data['tag_name'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Tag with such name already exists'));
        }

        $leadsTagModel = $leadsTagMapper->save($leadsTagModel);

        $this->_responseHelper->success(array('tagInfo' => $leadsTagModel->toArray(), 'message' =>$translator->translate(Leads::TRANSLATION_PREFIX.'Tag has been created')));
    }

    /**
     * Update lead tag data
     *
     * Resource:
     * : /api/leads/leadtags/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (source integer)
     * : lead tag id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            $translator = Zend_Registry::get('Zend_Translate');
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $leadTagId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadTagsMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
            $leadTagsModel = $leadTagsMapper->find($leadTagId);
            if (!$leadTagsModel instanceof Leads_Model_LeadsTagModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Config doesn\'t exists'));
            }

            $data['tag_name'] = rtrim(preg_replace('~[^\w\s\_]~ui', '', $data['tag_name']));
            $data['label'] = preg_replace('~[^\w\s\_]~ui', '', $data['label']);
            $oldTag = $leadTagsModel->getTagName();
            $currentTag = $data['tag_name'];
            if ($oldTag !== $currentTag) {
                $validateTagExists = new Zend_Validate_Db_RecordExists(array(
                    'table' => 'plugin_leads_tags',
                    'field' => 'tag_name'
                ));
                if ($validateTagExists->isValid($currentTag)) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You have another tag with such name'));
                }
            }

            $leadTagsModel->setOptions($data);
            $leadTagsMapper->save($leadTagsModel);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Tag has been updated'));
        }

    }

    /**
     * Delete lead tag
     *
     * Resource:
     * : /api/leads/leadtags/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (source integer)
     * : lead tag id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsTagsMapper = Leads_Mapper_LeadsTagsMapper::getInstance();
        $leadTagsModel = $leadsTagsMapper->find($id);
        if ($leadTagsModel instanceof Leads_Model_LeadsTagModel) {
            $leadsTagsMapper->delete($id);
        } else {
            $this->_error();
        }

    }

}
