<?php

class Api_Leads_Leadtype extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const LEADS_SECURE_TOKEN = 'LeadsToken';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('typeName', 'typeLabel');

    /**
     * Exclude from the config system values list
     *
     * @return array
     */
    public static function getExcludeSystemValues()
    {
        return array('inbound_quote_view', 'outbound_meeting');
    }

    /**
     * Exclude system activities values list
     */
    public static function getExcludeActivitiesValues()
    {
        //outbound_email, outbound_call, outbound_meeting
        return array(
            'inbound_email',
            'inbound_call',
            'inbound_quote',
            'inbound_purchase',
            'inbound_quote_view',
            'inbound_registration',
            'inbound_form',
            'inbound_downloader',
            'inbound_website_visit',
            'outbound_quote',
            'inbound_chat',
            'inbound_sms',
            'outbound_sms',
        );
    }

    /**
     * System statuses
     *
     * @return array
     */
    public static function systemStatuses()
    {
        return array(
            'inbound_email',
            'inbound_call',
            'inbound_quote',
            'inbound_purchase',
            'inbound_quote_view',
            'inbound_registration',
            'inbound_form',
            'inbound_downloader',
            'inbound_website_visit',
            'outbound_email',
            'outbound_call',
            'outbound_quote',
            'outbound_meeting',
            'inbound_chat',
            'inbound_sms',
            'outbound_sms'
        );
    }

    /**
     * Ordering values for Leads filter Order by dropdown
     *
     * @return array
     */
    public static function orderingDateAndInteractionValues()
    {
        $translator = Zend_Registry::get('Zend_Translate');
        return array(
            Api_Leads_Leads::SORT_ORDER_CREATED_AT => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead creation date'),
            Api_Leads_Leads::SORT_ORDER_LAST_ACTIVITY_INBOUND => $translator->translate(Leads::TRANSLATION_PREFIX.'Most recently engaged'),
            Api_Leads_Leads::SORT_ORDER_LAST_ACTIVITY_OUTBOUND => $translator->translate(Leads::TRANSLATION_PREFIX.'Last outbound action'),
            Api_Leads_Leads::SORT_ORDER_UPDATED_AT => $translator->translate(Leads::TRANSLATION_PREFIX.'Last record update'),
            Api_Leads_Leads::SORT_ORDER_LEAD_NAME => $translator->translate(Leads::TRANSLATION_PREFIX.'Lead name')
        );
    }

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Shopping::ROLE_SALESPERSON => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     * Get lead type by id
     *
     * Resource:
     * : /api/leads/leadtype/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {

        $limit = filter_var($this->_request->getParam('limit'), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset'), FILTER_SANITIZE_NUMBER_INT);
        $sortOrder = filter_var($this->_request->getParam('order', 'plt.type_name'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        $excludeSystemValues = filter_var($this->_request->getParam('excludeSystemValues'), FILTER_SANITIZE_STRING);

        $leadTypeDataMapper = Leads_Mapper_LeadsTypeMapper::getInstance();
        if ($id) {
            $where = $leadTypeDataMapper->getDbTable()->getAdapter()->quoteInto('plt.id = ?', $id);
            $data = $leadTypeDataMapper->fetchAllData($where, null, null, null, true);
        } else {
            $where = null;
            if (!empty($excludeSystemValues)) {
                $where =  $leadTypeDataMapper->getDbTable()->getAdapter()->quoteInto('plt.type_name NOT IN (?)', self::getExcludeSystemValues());
            }
            $data = $leadTypeDataMapper->fetchAllData($where, $sortOrder, $limit, $offset);
        }

        return $data;
    }

    /**
     * Create new lead type
     *
     * Resource:
     * : /api/leads/leadtype/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Missing mandatory params'));
        }

        $secureToken = $this->getRequest()->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
        if (!$tokenValid) {
            $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
            $websiteUrl = $websiteHelper->getUrl();
            $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
        }

        $leadsTypeMapper = Leads_Mapper_LeadsTypeMapper::getInstance();
        $leadsTypeModel = new Leads_Model_LeadsTypeModel();
        $leadsTypeModel->setOptions($data);

        $validator = new Zend_Validate_Db_NoRecordExists(array(
            'table' => 'plugin_leads_type',
            'field' => 'type_name'
        ));

        if (!$validator->isValid($data['typeName'])) {
            $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'Lead type with such name already exists'));
        }

        $data = $leadsTypeMapper->save($leadsTypeModel);

        $this->_responseHelper->success(array('typeId' => $data->getId(), 'message' => $translator->translate(Leads::TRANSLATION_PREFIX.'Type has been created')));
    }

    /**
     * Update lead type data
     *
     * Resource:
     * : /api/leads/leadtype/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     * : lead type id to update
     *
     * @return JSON
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        $translator = Zend_Registry::get('Zend_Translate');
        if (!empty($data['id']) && !empty($data[Tools_System_Tools::CSRF_SECURE_TOKEN])) {
            if (empty($data['type_label'])) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Label can\'t be empty'));
            }
            $secureToken = $data[Tools_System_Tools::CSRF_SECURE_TOKEN];
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::LEADS_SECURE_TOKEN);
            if (!$tokenValid) {
                $websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
                $websiteUrl = $websiteHelper->getUrl();
                $this->_error($translator->translate('Your session has timed-out. Please Log back in '.'<a href="'.$websiteUrl.'go">here</a>'));
            }
            $leadTypeId = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
            $leadTypeMapper = Leads_Mapper_LeadsTypeMapper::getInstance();
            $leadTypeModel = $leadTypeMapper->find($leadTypeId);
            if (!$leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
                $this->_responseHelper->fail($translator->translate(Leads::TRANSLATION_PREFIX.'Config doesn\'t exists'));
            }

            $oldType = $leadTypeModel->getTypeName();
            $currentType = $data['type_name'];
            if ($oldType !== $currentType) {
                $validateTypeExists = new Zend_Validate_Db_RecordExists(array(
                    'table' => 'plugin_leads_type',
                    'field' => 'type_name'
                ));
                if ($validateTypeExists->isValid($currentType)) {
                    $this->_error($translator->translate(Leads::TRANSLATION_PREFIX.'You have another lead type with such name'));
                }
            }

            if (!empty($data['default']) && $data['default'] === '1') {
                $leadTypeMapper->disableDefaultType();
                $leadTypeMapper->assignDefaultType($leadTypeId);
            }

            $leadTypeModel->setOptions($data);
            $leadTypeMapper->save($leadTypeModel);
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Lead type has been updated'));
        } else {
            $leadTypeMapper = Leads_Mapper_LeadsTypeMapper::getInstance();
            $leadTypeMapper->disableDefaultType();
            $this->_responseHelper->success($translator->translate(Leads::TRANSLATION_PREFIX.'Lead type has been updated'));
        }

    }

    /**
     * Delete lead type
     *
     * Resource:
     * : /api/leads/leadtype/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     * : lead type id to delete
     *
     * @return JSON
     */
    public function deleteAction()
    {
        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);
        if (!$id) {
            $this->_error();
        }
        $leadsTypeMapper = Leads_Mapper_LeadsTypeMapper::getInstance();
        $leadTypeModel = $leadsTypeMapper->find($id);
        if ($leadTypeModel instanceof Leads_Model_LeadsTypeModel) {
            $leadsTypeMapper->delete($id);
        } else {
            $this->_error();
        }

    }

}
