<?php

/**
 * LeadsEventsMapper.php
 *
 *
 * @method Leads_Mapper_LeadsEventsMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsEventsMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsEventModel';

    protected $_dbTable = 'Leads_DbTable_LeadsEventsDbTable';

    /**
     * Save lead event to DB
     * @param $model Leads_Model_LeadsEventModel
     * @return Leads_Model_LeadsEventModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'event_name' => $model->getEventName(),
            'event_label' => $model->getEventLabel()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        $this->processExternalEvents($id, $model->getEventExternalNames());

        return $model;
    }


    /**
     * Add external event ids to lead
     *
     * @param int $eventId lead id
     * @param array $externalIds tag ids
     */
    public function processExternalEvents($eventId, $externalIds)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('event_id = ?', $eventId);
        if (!empty($externalIds) && is_array($externalIds)) {
            $this->getDbTable()->getAdapter()->delete('plugin_leads_events_relation', $where);
            foreach ($externalIds as $externalId) {
                $data = array(
                    'event_id' => $eventId,
                    'external_id' => $externalId
                );
                $this->getDbTable()->getAdapter()->insert('plugin_leads_events_relation', $data);
            }
        }
    }

    /**
     * Get lead event data with info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount without count flag
     * @param bool $singleRecord if true return single record
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false
    ) {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('ple' => 'plugin_leads_events'),
                array(
                    'ple.id',
                    'ple.event_name',
                    'ple.event_label',
                    'event_external_names' => new Zend_Db_Expr('GROUP_CONCAT(pler.external_id)')
                )
            )->joinLeft(array('pler' => 'plugin_leads_events_relation'), 'pler.event_id=ple.id',
                array())->group('ple.id');

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord === true) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('ple' => 'plugin_leads_events'),
                array('count' => 'COUNT(ple.id)'))->joinLeft(array('pler' => 'plugin_leads_events_relation'),
                'pler.event_id=ple.id', array());
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        }

        return $data;
    }


    /**
     * Get all events pairs
     * @param array $order
     *
     * @return array
     */
    public function getEvents($order = array())
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_events',
            array('event_name', 'id', 'event_label'));

        if (!empty($order)) {
            $select->order($order);
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Get tag by event name
     *
     * @param string $eventName tag name
     *
     * @return array
     */
    public function getEventByName($eventName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('event_name = ?', $eventName);

        return $this->_findWhere($where);
    }

    /**
     * Get event ids by event external  id
     *
     * @param string $externalId external id(name)
     *
     * @return array
     */
    public function getEventByExternalId($externalId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('pler.external_id = ?', $externalId);
        $select = $this->getDbTable()->getAdapter()->select()->from(array('pler' => 'plugin_leads_events_relation'))->where($where);

        return $this->getDbTable()->getAdapter()->fetchPairs($select);

    }

    /**
     * Delete lead event record
     *
     * @param int $id lead event id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_events', $where);

    }

    /**
     * Delete event externals record
     *
     * @param int $id lead tag id
     * @return mixed
     * @throws Exception
     */
    public function deleteEventExternals($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('event_id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_events_relation', $where);

    }


}
