<?php

/**
 * LeadsEventsRulesGeneralConfigMapper.php
 *
 *
 * @method Leads_Mapper_LeadsEventsRulesGeneralConfigMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsEventsRulesGeneralConfigMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsEventsRulesGeneralConfigModel';

    protected $_dbTable = 'Leads_DbTable_LeadsEventsRulesGeneralConfigDbTable';

    /**
     * Save lead event general config model to DB
     * @param $model Leads_Model_LeadsEventsRulesGeneralConfigModel
     * @return Leads_Model_LeadsEventsRulesGeneralConfigModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'event_id' => $model->getEventId(),
            'rule_name' => $model->getRuleName(),
            'created_at' => $model->getCreatedAt(),
            'creator_id' => $model->getCreatorId(),
            'updated_at' => $model->getUpdatedAt(),
            'editor_id' => $model->getEditorId()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get event rules data
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = false
    ) {

        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = 'plergc.created_at';

        if ($withTimezoneOffset === true) {
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(`plergc`.`created_at`, '+00:00', '$mysqlOffset')");
        }

        $updatedAt = 'plergc.updated_at';

        if ($withTimezoneOffset === true) {
            $updatedAt = new Zend_Db_Expr("CONVERT_TZ(`plergc`.`updated_at`, '+00:00', '$mysqlOffset')");
        }

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plergc' => 'plugin_leads_event_rules_general_config'),
                array(
                    'plergc.id',
                    'plergc.event_id',
                    'ple.event_name',
                    'plergc.rule_name',
                    'plergc.creator_id',
                    'createdAt' => $createdAt,
                    'plergc.editor_id',
                    'updatedAt' => $updatedAt,
                    'creatorName' => 'u.full_name',
                    'editorName' => 'uc.full_name',
                    'actionTypes' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(plera.action_type))')
                )
            )
            ->joinLeft(array('plera' => 'plugin_leads_event_rules_actions'), 'plera.rule_id=plergc.id', array())
            ->joinLeft(array('ple' => 'plugin_leads_events'), 'ple.id=plergc.event_id', array())
            ->joinLeft(array('u' => 'user'), 'u.id=plergc.creator_id', array())
            ->joinLeft(array('uc' => 'user'), 'uc.id=plergc.editor_id', array());


        $select->group('plergc.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(plergc.id))'));

            $select->from(array('plergc' => 'plugin_leads_event_rules_general_config'), $count)
                ->joinLeft(array('plera' => 'plugin_leads_event_rules_actions'), 'plera.rule_id=plergc.id', array())
                ->joinLeft(array('ple' => 'plugin_leads_events'), 'ple.id=plergc.event_id', array())
                ->joinLeft(array('u' => 'user'), 'u.id=plergc.creator_id', array())
                ->joinLeft(array('uc' => 'user'), 'uc.id=plergc.editor_id', array());


            $select =  $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get assigned event rules
     *
     * @param array $eventIds event ids
     * @return array
     */
    public function getEventRules($eventIds)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plergc.event_id IN (?)', $eventIds);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plergc' => 'plugin_leads_event_rules_general_config'),
            array('plergc.id', 'plergc.event_id'))
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Get assigned event rules ids
     *
     * @param string $eventIds form name
     * @return array
     */
    public function getEventRulesWithRules($eventIds)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plergc.event_id = ?', $eventIds);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plergc' => 'plugin_leads_event_rules_general_config'),
            array('uniqueKey' => new Zend_Db_Expr("CONCAT(plergc.id, '_', plerc.id)"), 'plergc.id', 'plerc.field_name', 'plerc.rule_comparison_operator', 'plerc.field_value'))
            ->join(array('plerc' => 'plugin_leads_event_rules_config'), 'plerc.rule_id=plergc.id', array())
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }


    /**
     * get event rules
     * @param int $ruleId rule id
     * @return array
     */
    public function getEventFieldsByRuleId($ruleId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plerc.rule_id = ?', $ruleId);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plerc' => 'plugin_leads_event_rules_config'),
            array('plerc.field_name', 'plerc.id', 'plerc.rule_comparison_operator', 'plerc.field_value'))
            ->where($where)->order('plerc.field_name');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * get event rules
     * @param int $ruleId rule id
     * @return array
     */
    public function getEventActionsByRuleId($ruleId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plera.rule_id = ?', $ruleId);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plera' => 'plugin_leads_event_rules_actions'),
            array('plera.action_type', 'plera.id', 'plera.action_config'))
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Check if rule already exists
     *
     * @param string $eventId event id
     * @param string $ruleName unique rule name
     * @return null
     */
    public function checkRuleExist($eventId, $ruleName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('event_id = ?', $eventId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('rule_name = ?', $ruleName);

        return $this->_findWhere($where);
    }

    /**
     * Set not assigned creator id
     *
     * @param int $creatorId creator id
     * @throws Zend_Db_Adapter_Exception
     */
    public function setNotAssignedCreator($creatorId)
    {
        $where = new Zend_Db_Expr('creator_id IS NULL');

        $data = array('creator_id' => $creatorId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_event_rules_general_config', $data, $where);
    }

    /**
     * Set not assigned editor id
     *
     * @param int $editorId editor id
     * @throws Zend_Db_Adapter_Exception
     */
    public function setNotAssignedEditor($editorId)
    {
        $where = new Zend_Db_Expr('editor_id IS NULL');

        $data = array('editor_id' => $editorId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_event_rules_general_config', $data, $where);
    }

    /**
     * Delete form rules general config record
     *
     * @param int $id config id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_event_rules_general_config', $where);

    }

}
