<?php

/**
 * Leads_Mapper_LeadsMeetingMapper.php
 *
 *
 * @method Leads_Mapper_LeadsMeetingMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsMeetingMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsMeetingModel';

    protected $_dbTable = 'Leads_DbTable_LeadsMeetingDbTable';

    /**
     * Save lead meeting to DB
     * @param $model Leads_Model_LeadsMeetingModel
     * @return Leads_Model_LeadsMeetingModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'lead_id' => $model->getLeadId(),
            'start_time_date' => $model->getStartTimeDate(),
            'end_time_date' => $model->getEndTimeDate(),
            'location' => $model->getLocation(),
            'invite_attendees' => $model->getInviteAttendees(),
            'period' => $model->getPeriod(),
            'subject' => $model->getSubject(),
            'details' => $model->getDetails(),
            'google_calendar' => $model->getGoogleCalendar(),
            'creator_id' => $model->getCreatorId(),
            'first_notification_date' => $model->getFirstNotificationDate(),
            'second_notification_date' => $model->getSecondNotificationDate(),
            'first_notification_sent_date' => $model->getFirstNotificationSentDate(),
            'second_notification_sent_date' => $model->getSecondNotificationSentDate(),
            'first_notification_is_sent' => $model->getFirstNotificationIsSent(),
            'second_notification_is_sent' => $model->getSecondNotificationIsSent(),
            'send_notification_sms' => $model->getSendNotificationSms(),
            'send_sms' => $model->getSendSms(),
            'meeting_type' => $model->getMeetingType()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }


    /**
     * Get meetings tasks
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @param string $groupBy mysql group by
     * @param string $having mysql having
     * @return array
     */
    public function fetchAllData($where = null, $order = null, $limit = null, $offset = null,  $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = true,
        $groupBy = '',
        $having = '')
    {
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');
        $dueDate = 'plm.start_time_date';

        if ($withTimezoneOffset === true) {
            $dueDate = new Zend_Db_Expr("CONVERT_TZ(`plm`.`start_time_date`, '+00:00', '$mysqlOffset')");
        }

        $selectFields = array(
            'plm.lead_id',
            'date' => $dueDate,
            'plm.creator_id',
            'title' => 'plm.subject',
            'firstName' =>'pld.lead_first_name',
            'lastName' => 'pld.lead_last_name'
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plm' => 'plugin_leads_meeting'),
                $selectFields
            )
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plm.lead_id', array());

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        if (!empty($groupBy)) {
            $select->group($groupBy);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('plm' => 'plugin_leads_meeting'), array('count' => 'COUNT(plm.id)'))
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plm.lead_id', array());
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }


    /**
     * Change creator id for call
     *
     * @param int $creatorIdFrom creator id from (system user id)
     * @param int $creatorIdTo creator id  to (system user id)
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeCreatorId($creatorIdFrom, $creatorIdTo)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('creator_id = ?', $creatorIdFrom);
        $data = array('creator_id' => $creatorIdTo);

        $this->getDbTable()->getAdapter()->update('plugin_leads_meeting', $data, $where);
    }

    /**
     * Change lead id
     *
     * @param array $leadIds lead ids
     * @param int $newLeadId new lead id
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadId($leadIds, $newLeadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id IN (?)', $leadIds);
        $data = array('lead_id' => $newLeadId);

        $this->getDbTable()->getAdapter()->update('plugin_leads_meeting', $data, $where);
    }

    /**
     * Delete lead meeting
     *
     * @param int $id lead meeting id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_meeting', $where);

    }

    public function getFirstMeetingsForNotifications()
    {
        $timeNow = Tools_System_Tools::convertDateFromTimezone('now');
        $where = $this->getDbTable()->getAdapter()->quoteInto('first_notification_date < ?', $timeNow);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('first_notification_is_sent  = ?', '0');
        $where .= ' AND '. new Zend_Db_Expr('first_notification_date IS NOT NULL');
        $select = $this->getDbTable()->getAdapter()->select()->from(array('plm' => 'plugin_leads_meeting'), array(
            'plm.id', 'plm.*'
        ))->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    public function getSecondMeetingsForNotifications()
    {
        $timeNow = Tools_System_Tools::convertDateFromTimezone('now');
        $where = $this->getDbTable()->getAdapter()->quoteInto('second_notification_date < ?', $timeNow);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('second_notification_is_sent  = ?', '0');
        $where .= ' AND '. new Zend_Db_Expr('second_notification_date IS NOT NULL');
        $select = $this->getDbTable()->getAdapter()->select()->from(array('plm' => 'plugin_leads_meeting'), array(
            'plm.id', 'plm.*'
        ))->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * @param int $id meeting id
     */
    public function updateFirstNotificationSent($id)
    {
        $timeNow = Tools_System_Tools::convertDateFromTimezone('now');
        $data = array('first_notification_is_sent' => '1', 'first_notification_sent_date' => $timeNow);
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
        $this->getDbTable()->update($data, $where);
    }

    /**
     * @param int $id meeting id
     */
    public function updateSecondNotificationSent($id)
    {
        $timeNow = Tools_System_Tools::convertDateFromTimezone('now');
        $data = array('second_notification_is_sent' => '1', 'second_notification_sent_date' => $timeNow);
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
        $this->getDbTable()->update($data, $where);
    }

}
