<?php

/**
 * LeadsNoteMapper.php
 *
 *
 * @method Leads_Mapper_LeadsNoteMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsNoteMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsNoteModel';

    protected $_dbTable = 'Leads_DbTable_LeadsNoteDbTable';

    /**
     * Save lead note model to DB
     * @param $model Leads_Model_LeadsNoteModel
     * @return Leads_Model_LeadsNoteModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'note' => $model->getNote(),
            'lead_id' => $model->getLeadId(),
            'date_created' => $model->getDateCreated(),
            'user_id' => $model->getUserId()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get lead notes data with info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @return array
     */
    public function fetchAllData($where = null, $order = null, $limit = null, $offset = null, $withTimezoneOffset = true)
    {
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');
        $dateCreated = 'pln.date_created';

        if ($withTimezoneOffset === true) {
            $dateCreated = new Zend_Db_Expr("CONVERT_TZ(`pln`.`date_created`, '+00:00', '$mysqlOffset')");
        }

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pln' => 'plugin_leads_notes'),
                array(
                    'pln.id',
                    'pln.note',
                    'pln.lead_id',
                    'date_created' => $dateCreated,
                    'pln.user_id',
                    'u.full_name'
                ))->joinLeft(array('u' => 'user'), 'u.id=pln.user_id', array()
            );

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        $data = $this->getDbTable()->getAdapter()->fetchAll($select);

        $select->reset(Zend_Db_Select::COLUMNS);
        $select->reset(Zend_Db_Select::FROM);
        $select->reset(Zend_Db_Select::LIMIT_OFFSET);
        $select->reset(Zend_Db_Select::GROUP);

        $select->from(array('pln' => 'plugin_leads_notes'), array('count' => 'COUNT(pln.id)'));
        $count = $this->getDbTable()->getAdapter()->fetchRow($select);

        return array(
            'totalRecords' => $count['count'],
            'data' => $data,
            'offset' => $offset,
            'limit' => $limit
        );
    }

    /**
     * Get lead notes data with info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @param string $groupBy mysql group by
     * @param string $having mysql having
     * @return array
     */
    public function fetchAllDataForDashboard($where = null, $order = null, $limit = null, $offset = null,  $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = true,
        $groupBy = '',
        $having = '')
    {
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');
        $dueDate = 'pln.date_created';

        if ($withTimezoneOffset === true) {
            $dueDate = new Zend_Db_Expr("CONVERT_TZ(`pln`.`date_created`, '+00:00', '$mysqlOffset')");
        }

        $selectFields = array(
            'pln.lead_id',
            'date' => $dueDate,
            'pln.user_id',
            'title' => 'pln.note',
            'firstName' =>'pld.lead_first_name',
            'lastName' => 'pld.lead_last_name'
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pln' => 'plugin_leads_notes'),
                $selectFields
            )
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=pln.lead_id', array());

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        if (!empty($groupBy)) {
            $select->group($groupBy);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('pln' => 'plugin_leads_notes'), array('count' => 'COUNT(pln.id)'))
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=pln.lead_id', array());
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get all notes pairs
     *
     * @return array
     */
    public function getNotes()
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_notes', array('id', 'note'));

        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }

    /**
     * Change lead id
     *
     * @param array $leadIds lead ids
     * @param int $newLeadId new lead id
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadId($leadIds, $newLeadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id IN (?)', $leadIds);
        $data = array('lead_id' => $newLeadId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_notes', $data, $where);
    }

    /**
     * Delete lead note record
     *
     * @param int $id note id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_notes', $where);

    }

}
