<?php

/**
 * LeadsOpportunityMapper.php
 *
 *
 * @method Leads_Mapper_LeadsOpportunityMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsOpportunityMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsOpportunityModel';

    protected $_dbTable = 'Leads_DbTable_LeadsOpportunityDbTable';

    /**
     * Save lead opportunity model to DB
     * @param $model Leads_Model_LeadsOpportunityModel
     * @return Leads_Model_LeadsOpportunityModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'opportunity_name' => $model->getOpportunityName(),
            'opportunity_stage_id' => $model->getOpportunityStageId(),
            'creator_id' => $model->getCreatorId(),
            'lead_id' => $model->getLeadId(),
            'opportunity_value' => $model->getOpportunityValue(),
            'expected_close_date' => $model->getExpectedCloseDate(),
            'created_at' => $model->getCreatedAt(),
            'change_stage_date' => $model->getChangeStageDate(),
            'opportunity_type_id' => $model->getOpportunityTypeId()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get opportunity by name, stage id, lead id
     *
     * @param string $opportunityName opportunity
     * @param int $stageId opportunity stage id
     * @param int $leadId lead id
     * @param int $excludeId exclude opportunity id
     * @return mixed
     */
    public function getOpportunityByOpportunityNameStageIdLeadId($opportunityName, $stageId, $leadId, $excludeId = 0)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('opportunity_name = ?', $opportunityName);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('opportunity_stage_id = ?', $stageId);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('lead_id = ?', $leadId);

        if (!empty($excludeId)) {
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('id <> ?', $excludeId);
        }

        return $this->_findWhere($where);
    }

    /**
     * Get all opportunities pairs
     *
     * @return array
     */
    public function getOpportunities()
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_opportunity', array('id', 'note'));

        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }


    /**
     * Get opportunities data with info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param string $groupBy mysql group by
     * @param string $having mysql having
     * @return array
     */
    public function getOpportunitiesForDashboard(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $groupBy = '',
        $having = '',
        $withTimezoneOffset = true
    ){
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');
        $expectedCloseDate = 'plo.expected_close_date';

        if ($withTimezoneOffset === true) {
            $expectedCloseDate = new Zend_Db_Expr("CONVERT_TZ(`plo`.`expected_close_date`, '+00:00', '$mysqlOffset')");
        }

        $selectFields = array(
            'plo.lead_id',
            'expectedCloseDate' => $expectedCloseDate,
            'plo.creator_id',
            'opportunityName' => 'plo.opportunity_name',
            'firstName' =>'pld.lead_first_name',
            'lastName' => 'pld.lead_last_name',
            'organizationName' => 'plorg.organization_name',
            'organizationId' => 'pld.organization_id',
            'oppStageName' => 'plos.description',
            'oppStageId' => 'plos.id',
            'oppId' => 'plo.id'
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plo' => 'plugin_leads_opportunity'),
                $selectFields
            )
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
            ->joinLeft(array('plorg' => 'plugin_lead_organization'), 'plorg.id=pld.organization_id', array());

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        if (!empty($groupBy)) {
            $select->group($groupBy);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('plo' => 'plugin_leads_opportunity'), array('count' => 'COUNT(plo.id)'))
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
                ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
                ->joinLeft(array('plorg' => 'plugin_lead_organization'), 'plorg.id=pld.organization_id', array());
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get opportunities data with info
     *
     * @param null $where
     * @return array
     */
    public function getOpportunitiesForOpportunitiesTab($where = null){

        $selectFields = array(
            'leadId'             => 'plo.lead_id',
            'expectedCloseDate'  => 'plo.expected_close_date',
            'opportunityValue'   => 'plo.opportunity_value',
            'creatorId'          => 'plo.creator_id',
            'name'               => 'plo.opportunity_name',
            'prefix'             => 'pld.prefix',
            'firstName'          => 'pld.lead_first_name',
            'lastName'           => 'pld.lead_last_name',
            'organizationName'   => 'plorg.organization_name',
            'organizationId'     => 'pld.organization_id',
            'oppStageName'       => 'plos.description',
            'opportunityStageId' => 'plos.id',
            'oppId'              => 'plo.id',
            'userFullName'       => 'u.full_name'
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plo' => 'plugin_leads_opportunity'),
                $selectFields
            )
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plo.lead_id', array())
            ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
            ->joinLeft(array('plorg' => 'plugin_lead_organization'), 'plorg.id=pld.organization_id', array())
            ->joinLeft(array('u' => 'user'), 'u.id=plo.creator_id', array());

        if (!empty($where)) {
            $select->where($where);
        }

        $data = $this->getDbTable()->getAdapter()->fetchAll($select);

        return $data;
    }

    /**
     * Get opportunities data with info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param string $groupBy mysql group by
     * @param string $having mysql having
     * @param string $minDate min date
     * @return array
     */
    public function getOpportunitiesForTimeline(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $groupBy = '',
        $having = '',
        $withTimezoneOffset = true,
        $minDate = false
    ){
        if ($withTimezoneOffset === true) {
            $mysqlOffset = Tools_System_Tools::getUtcOffset('P');
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(`ploq`.`created_at`, '+00:00', '$mysqlOffset')");
            $opportunityCreatedAt = new Zend_Db_Expr("CONVERT_TZ(`plo`.`created_at`, '+00:00', '$mysqlOffset')");
            $expiredAt = new Zend_Db_Expr("CONVERT_TZ(`sq`.`expires_at`, '+00:00', '$mysqlOffset')");
            $expiredCloseDate = new Zend_Db_Expr("CONVERT_TZ(`plo`.`expected_close_date`, '+00:00', '$mysqlOffset')");
        } else{
            $createdAt = 'ploq.created_at';
            $expiredAt = 'sq.expires_at';
            $expiredCloseDate = 'plo.expected_close_date';
            $opportunityCreatedAt = 'plo.created_at';
        }

        if ($minDate === true) {
            $expiredCloseDate = new Zend_Db_Expr('MIN(plo.expected_close_date)');
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'),
            array(
                'quoteOwnerName' => 'u.full_name',
                'cartTotal' => 'scs.total',
                'opportunity_id' => 'plo.id',
                'plo.opportunity_name',
                'created_at' => $createdAt,
                'plo.opportunity_stage_id',
                'plo.lead_id',
                'plos.description',
                'ploq.quote_id',
                'sq.title',
                'sq.status',
                'expired_at' => $expiredAt,
                'plo.opportunity_value',
                'expected_close_date' => $expiredCloseDate,
                'opportunityCreatedAt' => $opportunityCreatedAt
            ))
            ->joinLeft(array('plos' => 'plugin_leads_opportunity_stage'), 'plo.opportunity_stage_id=plos.id', array())
            ->joinLeft(array('ploq' => 'plugin_leads_opportunity_quote'), 'plo.id=ploq.opportunity_id', array())
            ->joinLeft(array('sq' => 'shopping_quote'), 'sq.id=ploq.quote_id', array())
            ->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.id=sq.cart_id', array())
            ->joinLeft(array('u' => 'user'), 'u.id=sq.creator_id', array())
            ->where($where);

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        if (!empty($groupBy)) {
            $select->group($groupBy);
        }

        if (!empty($having)) {
            $select->having($having);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);
            if (empty($groupBy)) {
                $select->reset(Zend_Db_Select::GROUP);
            }

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(plo.id))'));

            $select->from(array('plo' => 'plugin_leads_opportunity'), $count)
                ->joinLeft(array('plos' => 'plugin_leads_opportunity_stage'), 'plo.opportunity_stage_id=plos.id', array())
                ->joinLeft(array('ploq' => 'plugin_leads_opportunity_quote'), 'plo.id=ploq.opportunity_id', array())
                ->joinLeft(array('sq' => 'shopping_quote'), 'sq.id=ploq.quote_id', array())
                ->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.id=sq.cart_id', array())
                ->joinLeft(array('u' => 'user'), 'u.id=sq.creator_id', array())
                ->where($where);

            $select =  $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get lead data with info for groups etc..
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = true
    ) {

        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $expectedAt = 'plo.expected_close_date';

        if ($withTimezoneOffset === true) {
            $expectedAt = new Zend_Db_Expr("CONVERT_TZ(`plo`.`expected_close_date`, '+00:00', '$mysqlOffset')");
        }

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plo' => 'plugin_leads_opportunity'),
                array(
                    'plo.id',
                    'plo.opportunity_name',
                    'plo.opportunity_stage_id',
                    'plo.creator_id',
                    'plo.lead_id',
                    'plo.opportunity_value',
                    'expected_close_date'=> $expectedAt,
                    'opportunity_type_id'
                )
            );
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('plo' => 'plugin_leads_opportunity'), array('count' => 'COUNT(plo.id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Delete lead opportunity record
     *
     * @param int $id opportunity id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_opportunity', $where);

    }


    ////////////////////////////////stats section////////////////////////////////////
    ///////////////////////////////lead detailed view///////////////////////////////
    /**
     * Get stats data of opportunities for leads
     *
     * @param int $leadId lead id
     * @param array $opportunityStageIds quote status (1, 10, 20)
     * @return Exception
     */
    public function getLeadOpportunityQuoteStatSalesDetailedView($leadId = 0, $opportunityStageIds = array())
    {
        $where = '';
        if (!empty($leadId)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plo.lead_id = ?', $leadId);
        }

        if (!empty($quoteType)) {
            $where .= Tools_LeadTools::addAndToStatement($where).$this->getDbTable()->getAdapter()->quoteInto('plos.id IN (?)', $opportunityStageIds);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array('plos.id', 'count' => new Zend_Db_Expr('COUNT(DISTINCT(plo.id))'), 'total' => 'SUM(plo.opportunity_value)', 'oppCreatedAt' => new Zend_Db_Expr('MIN(plo.created_at)'), 'oppLastChangeDate' => new Zend_Db_Expr('MAX(plo.change_stage_date)')))
            ->joinLeft(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array())
            ->group('plos.id');

        if (!empty($where)) {
            $select->where($where);
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     * @param int $organizationId
     * @param array $opportunityStageIds
     * @return mixed
     */
    public function getCartStatsForOpportunityStageOrganization($organizationId = 0, $opportunityStageIds = array())
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('scs.status IN (?)', array(Models_Model_CartSession::CART_STATUS_COMPLETED, Models_Model_CartSession::CART_STATUS_SHIPPED, Models_Model_CartSession::CART_STATUS_DELIVERED));
        if (!empty($organizationId)) {
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plo.id = ?', $organizationId);
        }

        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plop.opportunity_stage_id IN (?)', $opportunityStageIds);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_lead_organization'),  array('count' => new Zend_Db_Expr('COUNT(scs.id)'), 'total' => 'SUM(scs.total)'))
            ->join(array('pld' => 'plugin_leads_data'), 'pld.organization_id=plo.id', array())
            ->join(array('plop' => 'plugin_leads_opportunity'), 'plop.lead_id=pld.id', array())
            ->join(array('ploq' => 'plugin_leads_opportunity_quote'), 'ploq.opportunity_id=plop.id', array())
            ->join(array('scs' => 'shopping_cart_session'), 'scs.id=ploq.cart_id', array());

        $select->where($where);

        $select->group('plo.id');

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * @param int $leadId lead id
     * @param array $opportunityStageIds opportunity stage ids
     * @return mixed
     */
    public function getCartStatsForOpportunityStage($leadId = 0, $opportunityStageIds = array())
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('scs.status IN (?)', array(Models_Model_CartSession::CART_STATUS_COMPLETED, Models_Model_CartSession::CART_STATUS_SHIPPED, Models_Model_CartSession::CART_STATUS_DELIVERED));
        if (!empty($leadId)) {
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('pld.id = ?', $leadId);
        }

        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plop.opportunity_stage_id IN (?)', $opportunityStageIds);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'), array('count' => 'COUNT(scs.id)', 'total' => 'SUM(scs.total)'))
            ->join(array('plop' => 'plugin_leads_opportunity'), 'plop.lead_id=pld.id', array())
            ->join(array('ploq' => 'plugin_leads_opportunity_quote'), 'ploq.opportunity_id=plop.id', array())
            ->join(array('scs' => 'shopping_cart_session'), 'scs.id=ploq.cart_id', array());

        $select->where($where);

        $select->group('pld.id');

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Get stats opportunity data for organization
     *
     * @param int $organizationId lead id
     * @param array $opportunityStageIds quote status (1, 10, 20)
     * @return Exception
     */
    public function getLeadOrganizationOpportunityQuoteStatSalesDetailedView($organizationId = 0, $opportunityStageIds = array())
    {
        $where = '';
        if (!empty($organizationId)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('plo.id = ?', $organizationId);
        }

        if (!empty($quoteType)) {
            $where .= Tools_LeadTools::addAndToStatement($where).$this->getDbTable()->getAdapter()->quoteInto('plos.id IN (?)', $opportunityStageIds);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_lead_organization'), array('plos.id', 'count' => new Zend_Db_Expr('COUNT(DISTINCT(plop.id))'), 'total' => 'SUM(plop.opportunity_value)'))
            ->join(array('pld' => 'plugin_leads_data'), 'pld.organization_id=plo.id', array())
            ->join(array('plop' => 'plugin_leads_opportunity'), 'plop.lead_id=pld.id', array())
            ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plop.opportunity_stage_id', array())
            ->group('plos.id');

        if (!empty($where)) {
            $select->where($where);
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     * Get all sales for organization
     *
     * @param int $organizationId lead id
     * @return mixed
     */
    public function getLeadOrganizationTotalSalesDetailedView($organizationId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('scs.status IN (?)', array(Models_Model_CartSession::CART_STATUS_COMPLETED, Models_Model_CartSession::CART_STATUS_SHIPPED, Models_Model_CartSession::CART_STATUS_DELIVERED));
        if (!empty($organizationId)) {
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plo.id = ?', $organizationId);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_lead_organization'),  array('count' => new Zend_Db_Expr('COUNT(scs.id)'), 'total' => 'SUM(scs.total)'))
            ->joinLeft(array('pld' => 'plugin_leads_data'), 'pld.organization_id=plo.id', array())
            ->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.user_id=pld.user_id', array());

        $select->where($where);

        $select->group('plo.id');

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Get all sales
     *
     * @param int $leadId lead id
     * @return mixed
     */
    public function getLeadTotalSalesDetailedView($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('scs.status IN (?)', array(Models_Model_CartSession::CART_STATUS_COMPLETED, Models_Model_CartSession::CART_STATUS_SHIPPED, Models_Model_CartSession::CART_STATUS_DELIVERED));
        if (!empty($leadId)) {
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('pld.id = ?', $leadId);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'), array('count' => 'COUNT(scs.id)', 'total' => 'SUM(scs.total)'))
            ->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.user_id=pld.user_id', array());

        $select->where($where);

        $select->group('pld.id');


        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    public function getOpportunitiesWhichNotSold($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.opportunity_stage_id = ?', '100');
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plo.lead_id = ?', $leadId);
        $select = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'))
            ->joinLeft(array('ploq' => 'plugin_leads_opportunity_quote'), 'ploq.opportunity_id=plo.id', array())
            ->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.id=ploq.cart_id', array());

        $select->where($where);
        $select->group('ploq.opportunity_id');
        $select->having(new Zend_Db_Expr("GROUP_CONCAT(scs.status) NOT RLIKE '(completed|shipped|delivered)'"));

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    public function getOpportunitiesOrganizationWhichNotSold($organizationId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.opportunity_stage_id = ?', '100');
        if (!empty($organizationId)) {
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plorg.id = ?', $organizationId);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plorg' => 'plugin_lead_organization'))
            ->join(array('pld' => 'plugin_leads_data'), 'pld.organization_id=plorg.id', array())
            ->join(array('plo' => 'plugin_leads_opportunity'), 'pld.id=plo.lead_id')
            ->joinLeft(array('ploq' => 'plugin_leads_opportunity_quote'), 'ploq.opportunity_id=plo.id', array())
            ->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.id=ploq.cart_id', array());

        $select->where($where);
        $select->group('ploq.opportunity_id');
        $select->having(new Zend_Db_Expr("GROUP_CONCAT(scs.status) NOT RLIKE '(completed|shipped|delivered)'"));

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }



    public function getOpportunitiesWithoutQuotes($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.opportunity_stage_id = ?', '100');
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plo.lead_id = ?', $leadId);
        $where .= ' AND '. new Zend_Db_Expr('ploq.opportunity_id IS NULL');
        $select = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'))
            ->joinLeft(array('ploq' => 'plugin_leads_opportunity_quote'), 'ploq.opportunity_id=plo.id', array())
            ->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.id=ploq.cart_id', array());

        $select->where($where);
        $select->group('plo.id');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    public function getOpportunitiesOrganizationWithoutQuotes($organizationId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.opportunity_stage_id = ?', '100');
        if (!empty($organizationId)) {
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plorg.id = ?', $organizationId);
        }
        $where .= ' AND '. new Zend_Db_Expr('ploq.opportunity_id IS NULL');
        $select = $this->getDbTable()->getAdapter()->select()->from(array('plorg' => 'plugin_lead_organization'))
            ->join(array('pld' => 'plugin_leads_data'), 'pld.organization_id=plorg.id', array())
            ->join(array('plo' => 'plugin_leads_opportunity'), 'pld.id=plo.lead_id')
            ->joinLeft(array('ploq' => 'plugin_leads_opportunity_quote'), 'ploq.opportunity_id=plo.id', array())
            ->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.id=ploq.cart_id', array());

        $select->where($where);
        $select->group('plo.id');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     *  Get first purchase date
     *
     * @param int $leadId lead id
     * @return mixed
     */
    public function getFirstPurchaseDate($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('scs.status IN (?)', array(Models_Model_CartSession::CART_STATUS_COMPLETED, Models_Model_CartSession::CART_STATUS_SHIPPED, Models_Model_CartSession::CART_STATUS_DELIVERED));
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('scs.created_at <> ?', '0000-00-00 00:00:00');
        if (!empty($leadId)) {
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('pld.id = ?', $leadId);
        }
        $select = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'), array('scs.created_at'))
            ->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.user_id=pld.user_id', array());

        $select->where($where);
        $select->order('scs.created_at ASC');

        return $this->getDbTable()->getAdapter()->fetchRow($select);

    }

    ///////////////////////////////lead general stats///////////////////////////////
    /**
     * Get lead total sales
     *
     * @param array $leadIds lead ids
     * @return leads with stats
     */
    public function getLeadTotalSales(array $leadIds) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('scs.status IN (?)', array(Models_Model_CartSession::CART_STATUS_COMPLETED, Models_Model_CartSession::CART_STATUS_SHIPPED, Models_Model_CartSession::CART_STATUS_DELIVERED));
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('pld.id IN (?)', $leadIds);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('pld' => 'plugin_leads_data'), array('pld.id', 'count' => 'COUNT(scs.id)', 'total' => 'SUM(scs.total)'))
            ->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.user_id=pld.user_id', array());

        $select->where($where);
        $select->group('pld.id');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Get lead organization total sales
     *
     * @param array $organizationIds lead ids
     * @return leads with stats
     */
    public function getLeadOrganizationTotalSales(array $organizationIds) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('scs.status IN (?)', array(Models_Model_CartSession::CART_STATUS_COMPLETED, Models_Model_CartSession::CART_STATUS_SHIPPED, Models_Model_CartSession::CART_STATUS_DELIVERED));
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plo.id IN (?)', $organizationIds);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_lead_organization'), array('plo.id', 'count' => 'COUNT(scs.id)', 'total' => 'SUM(scs.total)'))
            ->join(array('pld' => 'plugin_leads_data'), 'pld.organization_id=plo.id', array())
            ->joinLeft(array('scs' => 'shopping_cart_session'), 'scs.user_id=pld.user_id', array());

        $select->where($where);
        $select->group('plo.id');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Get lead total opportunities
     *
     * @param array $leadIds lead ids
     * @param bool $withDate filter using dates
     * @param string $dateFrom date from
     * @param string $dateTo date to
     * @param string $dateType date type created/changed,expected
     * @param string $leadOpportunityType opportunity type
     * @param array $leadOpportunityStages opportunity stages
     * @param int $leadOpportunityOwner opportunity cretor id
     * @return array
     */
    public function getLeadTotalOpportunities(array $leadIds, $withDate  = false, $dateFrom = '', $dateTo = '', $dateType = 'created', $leadOpportunityType = '', $leadOpportunityStages = array(), $leadOpportunityOwner = 0)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.lead_id IN (?)', $leadIds);

        if ($withDate === true) {
            if ($dateType === 'created') {
                $dateFieldName = 'created_at';
            }

            if ($dateType === 'changed') {
                $dateFieldName = 'change_stage_date';
            }

            if ($dateType === 'expected') {
                $dateFieldName = 'expected_close_date';
            }

            if (!empty($dateFrom)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.'.$dateFieldName.' >= ?', $dateFrom);
            }
            if (!empty($dateTo)) {
                $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.'.$dateFieldName.' <= ?', $dateTo);
            }
        }

        if (!empty($leadOpportunityStages)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.opportunity_stage_id IN (?)', $leadOpportunityStages);
        }

        if (!empty($leadOpportunityType)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.opportunity_type_id = ?', $leadOpportunityType);
        }

        if (!empty($leadOpportunityOwner)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plo.creator_id = ?', $leadOpportunityOwner);
        }

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_leads_opportunity'), array('plo.lead_id', 'count' => new Zend_Db_Expr('COUNT(DISTINCT(plo.id))'), 'total' => 'SUM(plo.opportunity_value)'))
            ->joinLeft(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plo.opportunity_stage_id', array());

        $select->where($where);
        $select->group('plo.lead_id');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Get lead organization total opportunities
     *
     * @param array $organizationIds lead ids
     * @return array
     */
    public function getLeadOrganizationTotalOpportunities(array $organizationIds)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plo.id IN (?)', $organizationIds);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plo' => 'plugin_lead_organization'), array('plo.id', 'count' => new Zend_Db_Expr('COUNT(DISTINCT(plop.id))'), 'total' => 'SUM(plop.opportunity_value)'))
            ->join(array('pld' => 'plugin_leads_data'), 'pld.organization_id=plo.id', array())
            ->join(array('plop' => 'plugin_leads_opportunity'), 'plop.lead_id=pld.id', array())
            ->join(array('plos' => 'plugin_leads_opportunity_stage'), 'plos.id=plop.opportunity_stage_id', array());

        $select->where($where);
        $select->group('plo.id');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Change creator id for call
     *
     * @param int $creatorIdFrom creator id from (system user id)
     * @param int $creatorIdTo creator id  to (system user id)
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeCreatorId($creatorIdFrom, $creatorIdTo)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('creator_id = ?', $creatorIdFrom);
        $data = array('creator_id' => $creatorIdTo);
        $this->getDbTable()->getAdapter()->update('plugin_leads_opportunity', $data, $where);
    }

    /**
     * Change lead id
     *
     * @param array $leadIds lead ids
     * @param int $newLeadId new lead id
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadId($leadIds, $newLeadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id IN (?)', $leadIds);
        $data = array('lead_id' => $newLeadId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_opportunity', $data, $where);
    }

    /**
     * Get opportunity owners info
     * @param bool $pairs return result in pairs key => value
     *
     * @return array
     */
    public function getOpportunityOwners($pairs = false)
    {
        $select = $this->getDbTable()->getAdapter()->select()->from(array('plop' => 'plugin_leads_opportunity'),
            array('u.id', 'u.full_name'))->joinLeft(array('u' => 'user'), 'u.id=plop.creator_id')->group('u.id')->where('u.id IS NOT NULL')->order('u.full_name');

        if ($pairs ===true) {
            return $this->getDbTable()->getAdapter()->fetchPairs($select);
        } else {
            return $this->getDbTable()->getAdapter()->fetchAssoc($select);
        }
    }

    /**
     * Change opportunity stage
     *
     * @param $stageIdFrom
     * @param $stageIdTo
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeStageId($stageIdFrom, $stageIdTo, $opportunityId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('opportunity_stage_id = ?', $stageIdFrom);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('id = ?', $opportunityId);

        $data = array('opportunity_stage_id' => $stageIdTo);
        $this->getDbTable()->getAdapter()->update('plugin_leads_opportunity', $data, $where);
    }

}
