<?php

/**
 * LeadsTagsMapper.php
 *
 *
 * @method Leads_Mapper_LeadsTagsMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsTagsMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsTagModel';

    protected $_dbTable = 'Leads_DbTable_LeadsTagsDbTable';

    /**
     * Save lead tag to DB
     * @param $model Leads_Model_LeadsTagModel
     * @return Leads_Model_LeadsTagModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'tag_name' => $model->getTagName(),
            'label' => $model->getLabel()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get lead tag data with info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount without count flag
     * @param bool $singleRecord if true return single record
     * @return array
     */
    public function fetchAll($where = null, $order = null, $limit = null, $offset = null, $withoutCount = false, $singleRecord = false)
    {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plt' => 'plugin_leads_tags'),
                array(
                    'plt.id',
                    'plt.tag_name',
                    'plt.label'
                )
            );
        
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord === true) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('plt' => 'plugin_leads_tags'), array('count' => 'COUNT(plt.id)'));
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        }

        return $data;
    }


    /**
     * Get all tags pairs
     * @param array $order
     *
     * @return array
     */
    public function getTags($order = array())
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_tags', array('id', 'tag_name', 'label'));

        if (!empty($order)) {
            $select->order($order);
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }


    /**
     * Get all tags pairs
     * @param array $order
     *
     * @return array
     */
    public function getTagsPairs($order = array())
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_tags', array('id', 'label'));

        if (!empty($order)) {
            $select->order($order);
        }

        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }

    /**
     * Get all tags pairs
     *
     * @return array
     */
    public function getTagsForApi()
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_tags', array('id', 'label'));

        if (!empty($order)) {
            $select->order($order);
        }

        return $this->getDbTable()->getAdapter()->fetchAll($select);
    }

    /**
     * Get all tags for specified lead
     * @param int $leadId lead id
     *
     * @return array
     */
    public function getTagsForLeadApi($leadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plht.lead_id = ?', $leadId);
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plht' => 'plugin_leads_has_tags'), array('plt.id', 'plt.label'))
            ->join(array('plt' => 'plugin_leads_tags'), 'plht.tag_id=plt.id', array())
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchAll($select);
    }

    /**
     * Get tag by tag name
     *
     * @param string $tagName tag name
     *
     * @return array
     */
    public function getTag($tagName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('tag_name = ?', $tagName);

        return $this->_findWhere($where);
    }

    /**
     * Delete lead tag record
     *
     * @param int $id lead tag id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_tags', $where);

    }

    /**
     * @param $leadId
     * @return array
     */
    public function findTagsByLeadId($leadId){
        $where = $this->getDbTable()->getAdapter()->quoteInto('ld.id = ?', $leadId);

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('lt' => 'plugin_leads_tags'), array(
                'tagName'  => 'lt.tag_name',
                'tagId'  => 'lt.id'
            ))
            ->joinLeft(array('lht' => 'plugin_leads_has_tags'), 'lht.tag_id = lt.id', array())
            ->joinLeft(array('ld' => 'plugin_leads_data'), 'ld.id = lht.lead_id', array())
            ->where($where);
        $result = $this->getDbTable()->getAdapter()->fetchPairs($select);

        return $result;

    }

    /**
     * @param $tagNames
     * @return array
     */
    public function getAllIncomingTags($tagNames)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('tag_name IN (?)', $tagNames);

        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_tags', array('id', 'tag_name'))->where($where);

        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }

    /**
     * Get unused leads tags
     *
     * @return array
     */
    public function getUnusedTags() {
        $where = new Zend_Db_Expr('plht.tag_id IS NULL');

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plt' => 'plugin_leads_tags'),
                array(
                    'plt.id',
                    'plt.tag_name',
                    'plt.label'
                )
            )
            ->joinLeft(array('plht' => 'plugin_leads_has_tags'),
                'plht.tag_id = plt.id', array())
            ->where($where)
            ->group('plt.id');

        $data = $this->getDbTable()->getAdapter()->fetchAll($select);

        return $data;
    }

}
