<?php

/**
 * LeadsTaskMapper.php
 *
 *
 * @method Leads_Mapper_LeadsTaskMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsTaskMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsTaskModel';

    protected $_dbTable = 'Leads_DbTable_LeadsTaskDbTable';

    /**
     * Save lead task model to DB
     * @param $model Leads_Model_LeadsTaskModel
     * @return Leads_Model_LeadsTaskModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'title' => $model->getTitle(),
            'user_id' => $model->getUserId(),
            'creator_id' => $model->getCreatorId(),
            'expired_at' => $model->getExpiredAt(),
            'due_date' => $model->getDueDate(),
            'notes' => $model->getNotes(),
            'lead_id' => $model->getLeadId(),
            'creation_type' => $model->getCreationType(),
            'task_external_id' => $model->getTaskExternalId(),
            'is_task_completed' => $model->getIsTaskCompleted(),
            'is_task_overdue' => $model->getIsTaskCompleted(),
            'task_completed_by' => $model->getTaskCompletedBy(),
            'completion_comment' => $model->getCompletionComment(),
            'is_task_start_email_sent' => $model->getisTaskStartEmailSent()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get all tasks
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @param string $groupBy mysql group by
     * @param string $having mysql having
     * @return array
     */
    public function fetchAllData($where = null, $order = null, $limit = null, $offset = null,  $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = true,
        $groupBy = '',
        $having = '')
    {
        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');
        $dueDate = 'plt.due_date';

        if ($withTimezoneOffset === true) {
            $dueDate = new Zend_Db_Expr("CONVERT_TZ(`plt`.`due_date`, '+00:00', '$mysqlOffset')");
        }

        $selectFields = array(
            'plt.lead_id',
            'date' => $dueDate,
            'plt.user_id',
            'plt.title',
            'firstName' =>'pld.lead_first_name',
            'lastName' => 'pld.lead_last_name',
            'taskId' => 'plt.id',
            'userEmail' => 'u.email',
            'creatorEmail' => 'u1.email'
        );

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plt' => 'plugin_leads_tasks'),
                $selectFields
            )
            ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plt.lead_id', array())
            ->joinLeft(array('u' => 'user'), 'u.id=plt.creator_id', array())
            ->joinLeft(array('u1' => 'user'), 'u1.id=plt.user_id', array());

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        if (!empty($groupBy)) {
            $select->group($groupBy);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::GROUP);

            $select->from(array('plt' => 'plugin_leads_tasks'), array('count' => 'COUNT(plt.id)'))
                ->join(array('pld' => 'plugin_leads_data'), 'pld.id=plt.lead_id', array());
            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get upcoming tasks from action log table
     *
     * @param int $hoursShift number of hours
     * @return array
     */
    public function getTasksFromLogInDate($hoursShift)
    {
        $defaultTimezone = date_default_timezone_get();
        if (empty($defaultTimezone)) {
            $toDate = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d H:i:s',
                strtotime('+' . $hoursShift . ' hour')));
            $toNow =  Tools_System_Tools::convertDateFromTimezone(date('Y-m-d H:i:s',
                strtotime('- 24 hour')));
        } else {
            $toDate = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d H:i:s',
                strtotime('+' . $hoursShift . ' hour')), $defaultTimezone, 'UTC');
            $toNow = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d H:i:s',
                strtotime('- 24 hour')), $defaultTimezone, 'UTC');
        }

        $where = $this->getDbTable()->getAdapter()->quoteInto('plal.created_at <= ?', $toDate);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plal.created_at >= ?', $toNow);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.event_type = ?',
                Leads_Model_LeadsActionLogModel::EVENT_LOG_UPCOMING);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('plal.lead_type = ?',
                Leads_Model_LeadsTypeModel::LEAD_OUTBOUND_TASK);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plal' => 'plugin_leads_action_log'), array(
            'plt.id',
            'plt.title',
            'plt.notes',
            'plt.lead_id',
            'plt.user_id',
            'plt.creator_id',
            'taskExpired' => 'plal.created_at'
        ))
            ->join(array('pltl' => 'plugin_leads_tasks_log'), 'pltl.action_log_id=plal.id', array())
            ->join(array('plt' => 'plugin_leads_tasks'), 'pltl.task_id=plt.id', array())
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * @param $hoursShift
     * @return array
     */
    public function getTasksPreDueDate($hoursShift)
    {
        $defaultTimezone = date_default_timezone_get();
        if (empty($defaultTimezone)) {
            $toDate = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d H:i:s',
                strtotime('+' . $hoursShift . ' hour')));
            $toNow =  Tools_System_Tools::convertDateFromTimezone(date('Y-m-d H:i:s',
                strtotime('- 24 hour')));
        } else {
            $toDate = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d H:i:s',
                strtotime('+' . $hoursShift . ' hour')), $defaultTimezone, 'UTC');
            $toNow = Tools_System_Tools::convertDateFromTimezone(date('Y-m-d H:i:s',
                strtotime('- 24 hour')), $defaultTimezone, 'UTC');
        }

        $where = $this->getDbTable()->getAdapter()->quoteInto('plt.due_date <= ?', $toDate);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plt.due_date >= ?', $toNow);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plt.is_task_completed <> ?', '1');
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plt.is_task_overdue <> ?', '1');

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plt' => 'plugin_leads_tasks'), array(
            'plt.id',
            'plt.title',
            'plt.notes',
            'plt.lead_id',
            'plt.user_id',
            'plt.creator_id',
            'taskExpired' => 'plt.due_date'
        ))->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     *
     * Get all overdue tasks
     *
     * @return array
     */
    public function getAllOverdueTasks()
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plt.is_task_completed <> ?', '1');
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('plt.is_task_overdue = ?', '1');

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plt' => 'plugin_leads_tasks'), array(
            'plt.id',
            'plt.title',
            'plt.notes',
            'plt.lead_id',
            'plt.user_id',
            'plt.creator_id',
            'taskExpired' => 'plt.due_date'
        ))->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }


    /**
     * Find task by task external id
     *
     * @param string $externalId task external id
     * @return null
     */
    public function findByTaskExternalId($externalId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('task_external_id = ?', $externalId);
        return $this->_findWhere($where);
    }

    /**
     * Change lead id
     *
     * @param array $leadIds lead ids
     * @param int $newLeadId new lead id
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadId($leadIds, $newLeadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id IN (?)', $leadIds);
        $data = array('lead_id' => $newLeadId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_tasks', $data, $where);
    }


    /**
     * Set not assigned creator id
     *
     * @param int $creatorId creator id
     * @throws Zend_Db_Adapter_Exception
     */
    public function setNotAssignedCreator($creatorId)
    {
        $where = new Zend_Db_Expr('creator_id IS NULL');

        $data = array('creator_id' => $creatorId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_tasks', $data, $where);
    }

    /**
     * Set not assigned user id
     *
     * @param int $userId user id
     * @throws Zend_Db_Adapter_Exception
     */
    public function setNotAssignedUser($userId)
    {
        $where = new Zend_Db_Expr('user_id IS NULL');

        $data = array('user_id' => $userId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_tasks', $data, $where);
    }



    /**
     * Delete lead record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_tasks', $where);

    }

    /**
     * Close task.
     *
     * @param int $taskId task id
     * @param int $userId completed by user id
     * @param string $completionMessage completion message
     */
    public function closeTask($taskId, $userId, $completionMessage)
    {
        if (!empty($taskId)) {
            $data = array('is_task_completed' => '1', 'task_completed_by' => $userId, 'completion_comment' => $completionMessage);
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $taskId);
            $this->getDbTable()->update($data, $where);
        }
    }

    /**
     * Open task
     *
     * @param int $taskId task id
     * @param int $userId completed by user id
     * @param string $completionMessage completion message
     */
    public function openTask($taskId, $userId, $completionMessage)
    {
        if (!empty($taskId)) {
            $data = array('is_task_completed' => '0', 'task_completed_by' => $userId, 'completion_comment' => $completionMessage);
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $taskId);
            $this->getDbTable()->update($data, $where);
        }
    }

    /**
     * mark tasks overdue
     */
    public function markTasksAsOverdue()
    {
        $data = array('is_task_overdue' => '1');
        $now = Tools_System_Tools::convertDateFromTimezone('now');
        $where = $this->getDbTable()->getAdapter()->quoteInto('is_task_completed = ?', '0');
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('due_date <= ?', $now);
        $this->getDbTable()->update($data, $where);
    }


    /**
     * Get not sent task start time
     *
     * @param string $beforeStartTime
     * @param string $beforeTimeLimit
     * @return array
     */
    public function getNotSentTaskStartEmails($beforeTimeLimit, $beforeStartTime)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('is_task_start_email_sent = ?', '0');
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('expired_at >= ?', $beforeStartTime);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('expired_at <= ?', $beforeTimeLimit);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plt' => 'plugin_leads_tasks'), array(
            'plt.id', 'plt.lead_id'
        ))->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * mark tasks overdue
     * @param int $taskId task id
     */
    public function setStatusStartSent($taskId)
    {
        $data = array('is_task_start_email_sent' => '1');
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $taskId);
        $this->getDbTable()->update($data, $where);
    }


}
