<?php

/**
 * LeadsTaskQueueMapper.php
 *
 *
 * @method Leads_Mapper_LeadsTaskQueueMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsTaskQueueMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsTaskQueueModel';

    protected $_dbTable = 'Leads_DbTable_LeadsTaskQueueDbTable';

    /**
     * Save lead task model to DB
     * @param $model Leads_Model_LeadsTaskQueueModel
     * @return Leads_Model_LeadsTaskQueueModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'task_id' => $model->getTaskId(),
            'to_send_date' => $model->getToSendDate(),
            'sent_date' => $model->getSentDate(),
            'status' => $model->getStatus(),
            'queue_type' => $model->getQueueType()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Find task in queue by task id and queue type
     *
     * @param int $taskId real task id
     * @param string $queueType queue type (first_task_reminder|second_task_reminder|custom)
     * @return null
     */
    public function findByTaskIdQueueType($taskId, $queueType)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('task_id = ?', $taskId);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('queue_type = ?', $queueType);

        return $this->_findWhere($where);
    }

    /**
     * Get all not processed items for tasks queue
     *
     * @param int $limit sql query limit
     * @param array $types queue types
     * @return array
     */
    public function getAllQueue($limit = 0, $types = array())
    {
        $defaultTimezone = date_default_timezone_get();
        if (empty($defaultTimezone)) {
            $now = Tools_System_Tools::convertDateFromTimezone('now');
        } else {
            $now = Tools_System_Tools::convertDateFromTimezone('now', $defaultTimezone, 'UTC');
        }

        $where = $this->getDbTable()->getAdapter()->quoteInto('to_send_date < ?', $now);
        $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('status = ?',
                Leads_Model_LeadsTaskQueueModel::TASK_QUEUE_STATUS_PROCESSING);

        if (!empty($types)) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('queue_type IN (?)',
                    $types);
        }
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_tasks_queue',
            array('id', 'task_id'))->where($where);

        if (!empty($limit)) {
            $select->limit($limit);
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     * Update queue status
     *
     * @param int $queueId queue id
     * @param string $status queue status
     * @throws Zend_Db_Adapter_Exception
     */
    public function updateQueueStatus($queueId, $status)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $queueId);
        if (empty($defaultTimezone)) {
            $now = Tools_System_Tools::convertDateFromTimezone('now');
        } else {
            $now = Tools_System_Tools::convertDateFromTimezone('now', $defaultTimezone, 'UTC');
        }

        $data = array('status' => $status, 'sent_date' => $now);
        $this->getDbTable()->getAdapter()->update('plugin_leads_tasks_queue', $data, $where);

    }

    /**
     * Delete from queue by type
     *
     * @param string $queueType queue type
     * @param string $status status
     * @return int
     */
    public function deleteByQueueType($queueType, $status)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('queue_type = ?', $queueType);
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('status = ?', $status);
        return $this->getDbTable()->getAdapter()->delete('plugin_leads_tasks_queue', $where);
    }

    /**
     * Delete lead record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_tasks_queue', $where);

    }

}
