<?php

/**
 * LeadsTasksLogMapper.php
 *
 *
 * @method Leads_Mapper_LeadsTasksLogMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsTasksLogMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsTasksLogModel';

    protected $_dbTable = 'Leads_DbTable_LeadsTasksLogDbTable';

    /**
     * Save tasks log model to DB
     * @param $model Leads_Model_LeadsTasksLogModel
     * @return Leads_Model_LeadsTasksLogModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'title' => $model->getTitle(),
            'user_id' => $model->getUserId(),
            'creator_id' => $model->getCreatorId(),
            'expired_at' => $model->getExpiredAt(),
            'due_date' => $model->getDueDate(),
            'notes' => $model->getNotes(),
            'action_log_id' => $model->getActionLogId(),
            'task_id' => $model->getTaskId(),
            'lead_id' => $model->getLeadId(),
            'is_task_completed' => $model->getIsTaskCompleted(),
            'is_task_overdue' => $model->getIsTaskOverdue(),
            'task_completed_by' => $model->getTaskCompletedBy(),
            'completion_comment' => $model->getCompletionComment()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Find task by task id
     *
     * @param string $taskId task external id
     * @param bool $singleRecord return single record
     * @return null
     */
    public function findByTaskId($taskId, $singleRecord = false)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('task_id = ?', $taskId);
        if ($singleRecord === true) {
            return $this->_findWhere($where);
        }
        return $this->fetchAll($where);
    }


    /**
     * Delete tasks email log record
     *
     * @param int $id lead id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_tasks_log', $where);

    }


    /**
     * Close task.
     *
     * @param int $taskId task id
     * @param int $userId completed by user id
     * @param string $completionMessage completion message
     */
    public function closeTask($taskId, $userId, $completionMessage)
    {
        if (!empty($taskId)) {
            $data = array('is_task_completed' => '1', 'task_completed_by' => $userId, 'completion_comment' => $completionMessage);
            $where = $this->getDbTable()->getAdapter()->quoteInto('task_id = ?', $taskId);
            $this->getDbTable()->update($data, $where);
        }
    }

    /**
     * Open task
     *
     * @param int $taskId task id
     * @param int $userId completed by user id
     * @param string $completionMessage completion message
     */
    public function openTask($taskId, $userId, $completionMessage)
    {
        if (!empty($taskId)) {
            $data = array('is_task_completed' => '0', 'task_completed_by' => $userId, 'completion_comment' => $completionMessage);
            $where = $this->getDbTable()->getAdapter()->quoteInto('task_id = ?', $taskId);
            $this->getDbTable()->update($data, $where);
        }
    }



    /**
     * Find by type and task id
     *
     * @param string $type type (task, meeting, form etc..)
     * @param int $taskId task id
     * @return null
     */
    public function findByTypeTaskId($type, $taskId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('type = ?', $type);
        $where .= ' AND ' .$this->getDbTable()->getAdapter()->quoteInto('task_id = ?', $taskId);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('pltl' => 'plugin_leads_tasks_log'))
            ->joinLeft(array('plal' => 'plugin_leads_action_log'), 'pltl.action_log_id=plal.id', array())->where($where);

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    /**
     * Delete by type and task id
     *
     * @param string $type type (task, meeting, form etc..)
     * @param int $taskId task id
     * @return null
     */
    public function deleteByLeadTypeAndTaskId($type, $taskId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('type = ?', $type);
        $where .= ' AND ' .$this->getDbTable()->getAdapter()->quoteInto('task_id = ?', $taskId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_action_log', $where);
    }

    /** Change creator id for call
     *
     * @param int $creatorIdFrom creator id from (system user id)
     * @param int $creatorIdTo creator id  to (system user id)
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeCreatorId($creatorIdFrom, $creatorIdTo)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('creator_id = ?', $creatorIdFrom);
        $data = array('creator_id' => $creatorIdTo);
        $this->getDbTable()->getAdapter()->update('plugin_leads_tasks_log', $data, $where);
    }

    /** Change user id for call
     *
     * @param int $userIdFrom creator id from (system user id)
     * @param int $userIdTo creator id  to (system user id)
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeUserId($userIdFrom, $userIdTo)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('user_id = ?', $userIdFrom);
        $data = array('user_id' => $userIdTo);
        $this->getDbTable()->getAdapter()->update('plugin_leads_tasks_log', $data, $where);
    }


    /**
     * Deete by action log id
     *
     * @param int $logActionId action log id
     * @return int
     */
    public function deleteByActionLogId($logActionId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('action_log_id = ?', $logActionId);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_tasks_log', $where);
    }

    /**
     * Change lead id
     *
     * @param array $leadIds lead ids
     * @param int $newLeadId new lead id
     * @throws Zend_Db_Adapter_Exception
     */
    public function changeLeadId($leadIds, $newLeadId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('lead_id IN (?)', $leadIds);
        $data = array('lead_id' => $newLeadId);
        $this->getDbTable()->getAdapter()->update('plugin_leads_tasks_log', $data, $where);
    }

    /**
     * Mark tasks as overdue
     */
    public function markTasksAsOverdue()
    {
        $data = array('is_task_overdue' => '1');
        $now = Tools_System_Tools::convertDateFromTimezone('now');
        $where = $this->getDbTable()->getAdapter()->quoteInto('is_task_completed = ?', '0');
        $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('due_date <= ?', $now);
        $this->getDbTable()->update($data, $where);
    }

}
