<?php

/**
 * LeadsTypeMapper.php
 *
 *
 * @method Leads_Mapper_LeadsTypeMapper   getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsTypeMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsTypeModel';

    protected $_dbTable = 'Leads_DbTable_LeadsTypeDbTable';

    /**
     * Save lead type to DB
     * @param $model Leads_Model_LeadsTypeModel
     * @return Leads_Model_LeadsTypeModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'type_name' => $model->getTypeName(),
            'type_label' => $model->getTypeLabel(),
            'system' => $model->getSystem(),
            'default' => $model->getDefault()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get lead type data with info
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $singleRow single row result
     * @return array
     */
    public function fetchAllData($where = null, $order = null, $limit = null, $offset = null, $singleRow = false)
    {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plt' => 'plugin_leads_type'),
                array(
                    'plt.id',
                    'plt.type_name',
                    'plt.type_label',
                    'plt.system',
                    'plt.default'
                )
            );
        
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRow === true) {
            return  $this->getDbTable()->getAdapter()->fetchRow($select);
        }

        $data = $this->getDbTable()->getAdapter()->fetchAll($select);

        $select->reset(Zend_Db_Select::COLUMNS);
        $select->reset(Zend_Db_Select::FROM);
        $select->reset(Zend_Db_Select::LIMIT_OFFSET);
        $select->reset(Zend_Db_Select::GROUP);

        $select->from(array('plt' => 'plugin_leads_type'), array('count' => 'COUNT(plt.id)'));
        $count = $this->getDbTable()->getAdapter()->fetchRow($select);

        return array(
            'totalRecords' => $count['count'],
            'data' => $data,
            'offset' => $offset,
            'limit' => $limit
        );
    }

    /**
     * Find by lead type name
     *
     * @param string $typeName lead type name
     * @return null
     */
    public function findByTypeName($typeName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('type_name = ?', $typeName);
        return $this->_findWhere($where);
    }


    /**
     * Get all types pairs
     *
     * @param bool $systemOnly flag only system types
     * @param array $excludeStatuses exclude statuses
     * @param array $order order by
     * @param array $customColumns custom columns
     *
     * @return array
     */
    public function getTypes($systemOnly = false, $excludeStatuses = array(), $order = array(), $customColumns = array())
    {
        $columns = array('id', 'type_label');
        if (!empty($customColumns)) {
            $columns = $customColumns;
        }

        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_type', $columns);

        $where = '';
        if ($systemOnly === true) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('`system` = ?', '1');
        }

        if (!empty($excludeStatuses)) {
            if (!empty($where)) {
                $where .= ' AND ';
            }
            $where .= $this->getDbTable()->getAdapter()->quoteInto('type_name NOT IN (?)', $excludeStatuses);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($customColumns)) {
            return $this->getDbTable()->getAdapter()->fetchAssoc($select);
        }

        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }

    /**
     * Get all type names and labels pairs
     *
     * @return array
     */
    public function getTypesAndLabelsPairs()
    {
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_leads_type', array('type_name', 'type_label'));

        return $this->getDbTable()->getAdapter()->fetchPairs($select);
    }

    /**
     * Assign default type
     *
     * @param int $typeId type id
     * @throws Zend_Db_Adapter_Exception
     */
    public function assignDefaultType($typeId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $typeId);
        $data = array('default' => '1');
        $this->getDbTable()->getAdapter()->update('plugin_leads_type', $data, $where);

    }

    /**
     * Disable all default types
     *
     * @return int
     * @throws Zend_Db_Adapter_Exception
     */
    public function disableDefaultType()
    {
        return $this->getDbTable()->getAdapter()->update('plugin_leads_type', array('default' => '0'));
    }

    /**
     * get default type
     *
     * @return null
     */
    public function getDefaultType()
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('`default` = ?', '1');
        return $this->_findWhere($where);
    }


    /**
     * Delete lead type record
     *
     * @param int $id lead type id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_type', $where);

    }

}
