<?php

/**
 * Class Tools_LeadEmailTools
 */
class Tools_LeadEmailTools
{

    /**
     * Prepare email tracking link
     *
     * @return array
     * @throws Zend_Controller_Action_Exception
     */
    public static function prepareEmailTrackingLink()
    {
        $trackingCodeHash = hash('sha256', microtime() . rand(1, 1000));
        $websiteHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('website');
        $websiteUrl = $websiteHelper->getUrl();

        return array(
            'error' => 0,
            'message' => '',
            'trackingLink' => '<img src="' . $websiteUrl . 'plugin/leads/run/logPixelTrackingCode/trackingCode/' . $trackingCodeHash . '">',
            'trackingCodeHash' => $trackingCodeHash
        );
    }

    /**
     * Log email to email log
     *
     * @param int $leadId lead id
     * @param string $emailContent full email content
     * @param string $emailMessage email message
     * @param string $trackingCodeHash tracking hash code sha256
     * @param string $emailFrom email from
     * @param string $emailTo email to
     * @param string $emailSubject email subject
     * @param array $bccEmails bcc emails
     * @param string $createdAt email sent date
     * @param array $dictionary entity parser dictionary
     * @param object $observableModel observable model
     * @param int $sequenceId sequence id
     * @throws Zend_Exception
     * @return array
     */
    public static function logEmailSent(
        $leadId,
        $emailContent,
        $emailMessage,
        $trackingCodeHash,
        $emailFrom,
        $emailTo,
        $emailSubject,
        $bccEmails = array(),
        $createdAt = '',
        $dictionary = array(),
        $observableModel = null,
        $sequenceId = ''
    ) {
        if (empty($createdAt)) {
            $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
        }
        $actionLogModel = Tools_LeadTools::logLeadAction(Leads_Model_LeadsActionLogModel::ACTION_EMAIL, $leadId,
            Leads_Model_LeadsActionLogModel::ACTION_TYPE_OUTBOUND, $createdAt,
            Leads_Model_LeadsTypeModel::LEAD_TYPE_OUTBOUND_EMAIL, Leads_Model_LeadsActionLogModel::EVENT_LOG_EXPIRED,
            $emailContent);
        $optionsMessage = self::preparseOptionMessage($emailMessage, $dictionary, $observableModel);

        $leadsEmailLogMapper = Leads_Mapper_LeadsEmailLogMapper::getInstance();
        $leadsEmailLogModel = new Leads_Model_LeadsEmailLogModel();
        $leadsEmailLogModel->setActionLogId($actionLogModel->getId());
        $leadsEmailLogModel->setContent($optionsMessage);
        $leadsEmailLogModel->setEmailFrom($emailFrom);
        $leadsEmailLogModel->setEmailTo($emailTo);
        $leadsEmailLogModel->setEmailSubject($emailSubject);
        $leadsEmailLogModel->setEmailTemplate($emailContent);
        $leadsEmailLogModel->setTrackingCode($trackingCodeHash);
        $leadsEmailLogModel->setLeadId($leadId);
        if (!empty($sequenceId)) {
            $leadsEmailLogModel->setFollowUpSequencesId($sequenceId);
        }
        $leadsEmailLogModel->setEmailStatus(Leads_Model_LeadsEmailLogModel::EMAIL_STATUS_SENT);
        if (!empty($bccEmails)) {
            $leadsEmailLogModel->setEmailBcc(implode(',', $bccEmails));
        }
        $leadsEmailLogMapper->save($leadsEmailLogModel);

        return array('error' => 0, 'message' => '');
    }


    /**
     * Parse $optionMessage data before save LeadsEmailLog
     *
     * @param string $optionMessage
     * @param array $dictionary entity parser dictionary
     * @param object $observableModel observable model
     * @return null
     * @throws Zend_Exception
     */
    public static function preparseOptionMessage($optionMessage, array $dictionary, $observableModel)
    {

        $entityParser = new Tools_Content_EntityParser();
        $websiteHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('website');
        $entityParser->addToDictionary($dictionary);
        $entityParser->addToDictionary(
            array(
                'emailmessage' => !empty($optionMessage) ? $optionMessage : ''
            )
        );

        $mailTemplate = $entityParser->parse($optionMessage);

        $themeData = Zend_Registry::get('theme');
        $extConfig = Zend_Registry::get('extConfig');
        $parserOptions = array(
            'websiteUrl' => $websiteHelper->getUrl(),
            'websitePath' => $websiteHelper->getPath(),
            'currentTheme' => $extConfig['currentTheme'],
            'themePath' => $themeData['path'],
            'observableModel' => $observableModel
        );
        $parser = new Tools_Content_Parser($mailTemplate, array(), $parserOptions);

        $content = $parser->parse();

        return $content;
    }


    public static function addTrackableLinks($template, $trackingCodeHash = '', $skipRecording = false)
    {
        $websiteHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('website');
        $websiteUrl = $websiteHelper->getUrl();
        $doc = new DOMDocument();
        $doc->loadHTML(mb_convert_encoding('<html><body>'.$template.'</body></html>', 'HTML-ENTITIES', 'UTF-8'), LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
        $xpath = new DOMXpath($doc);
        if (empty($trackingCodeHash)) {
            $trackingCodeHash = hash('sha256', microtime() . rand(1, 1000));
        }
        $leadsEmailLinkAttachmentLogMapper = Leads_Mapper_LeadsEmailLinkAttachmentLogMapper::getInstance();
        $currentTime = Tools_System_Tools::convertDateFromTimezone('now');
        foreach ($xpath->query('//a') as $fileLinkEl) {
            $fileLink = $fileLinkEl->getAttribute('href');
            $linkText = $fileLinkEl->textContent;
            if (empty($linkText)) {
                $linkText = 'Link name not specified';
            }
            $isFile = false;
            $fileCodeHash = hash('sha256', $fileLink);
            //verification for a valid link
            if (filter_var($fileLink, FILTER_VALIDATE_URL) === false) {
                if (strpos($fileLink, $websiteUrl.'media/') === false) {
                    continue;
                }

                //verification for local files
                $fileLinkForFile = str_replace($websiteUrl.'media/', '', $fileLink);
                $filePathNameElements = explode('/', $fileLinkForFile);
                if (!empty($filePathNameElements[0]) && !empty($filePathNameElements[1])) {
                    $fileFolderName = $filePathNameElements[0];
                    $fileLinkName = $filePathNameElements[1];
                    $fileCodeHash = hash('sha256', $fileFolderName . $fileLinkName);
                    $isFile = true;
                } else {
                    continue;
                }
            }

            $trackingLink = $websiteUrl . 'plugin/leads/run/openLink/fileCode/' . $fileCodeHash . '/emailCode/' . $trackingCodeHash;
            $fileLinkEl->setAttribute('href', $trackingLink);
            $leadsEmailLinkAttachmentLogModel = $leadsEmailLinkAttachmentLogMapper->getByFileHashSentHash($trackingCodeHash, $fileCodeHash);
            if (!$leadsEmailLinkAttachmentLogModel instanceof Leads_Model_LeadsEmailLinkAttachmentLogModel) {
                $leadsEmailLinkAttachmentLogModel = new Leads_Model_LeadsEmailLinkAttachmentLogModel();
                $leadsEmailLinkAttachmentLogModel->setEmailLogSentHash($trackingCodeHash);
                $leadsEmailLinkAttachmentLogModel->setFileHash($fileCodeHash);
                if ($isFile === true) {
                    $leadsEmailLinkAttachmentLogModel->setFileName($fileLinkName);
                    $leadsEmailLinkAttachmentLogModel->setFileFolderName($fileFolderName);
                    $leadsEmailLinkAttachmentLogModel->setFileLink('');
                    $leadsEmailLinkAttachmentLogModel->setLinkType(Leads_Model_LeadsEmailLinkAttachmentLogModel::LINK_TYPE_ATTACHMENT);
                } else {
                    $leadsEmailLinkAttachmentLogModel->setFileName('');
                    $leadsEmailLinkAttachmentLogModel->setFileFolderName('');
                    $leadsEmailLinkAttachmentLogModel->setFileLink($fileLink);
                    $leadsEmailLinkAttachmentLogModel->setFileName($linkText);
                    $leadsEmailLinkAttachmentLogModel->setLinkType(Leads_Model_LeadsEmailLinkAttachmentLogModel::LINK_TYPE_CLICK);
                }
                $leadsEmailLinkAttachmentLogModel->setArchived('0');
                $leadsEmailLinkAttachmentLogModel->setCreatedDateTime($currentTime);
                $leadsEmailLinkAttachmentLogMapper->save($leadsEmailLinkAttachmentLogModel);
            }
        }

        $template = strtr($doc->saveHTML(), array('<html>' => '', '<body>'  => '', '</body>'  => '', '</html>' => ''));

        return $template;
    }

}