<?php

/**
 * Class Lead_LeadFormTools
 */
class Tools_LeadFormTools extends Tools_LeadFormEventsTools
{

    const ORGANIZATION_TYPE_CUSTOM_FIELD = 'organization';

    const LEAD_TYPE_CUSTOM_FIELD = 'lead';

    /**
     * @var array custom attributes types with predefined values
     * Such as select, date, checkbox
     */
    public static $_customAttributesTypesWithPredefinedValues = array(
        Tools_LeadImportTools::ATTRIBUTE_TYPE_SELECT,
        Tools_LeadImportTools::ATTRIBUTE_TYPE_CHECKBOX
    );

    /**
     * parse form field names
     *
     * @param string $formName form name
     * @return array
     */
    public static function getSystemFormFields($formName)
    {
        $formModel = Application_Model_Mappers_FormMapper::getInstance()->findByName($formName);
        if ($formModel instanceof Application_Model_Models_Form) {
            $formCode = $formModel->getCode();
            $doc = new DOMDocument();
            /**
             * LIBXML_HTML_NOIMPLIED turns off the automatic adding of implied html/body elements
             * LIBXML_HTML_NODEFDTD prevents a default doctype being added when one is not found.
             */
            $doc->loadHTML(mb_convert_encoding('<html><body>' . $formCode . '</body></html>', 'HTML-ENTITIES', 'UTF-8'),
                LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
            $xpath = new DOMXpath($doc);
            $fieldNames = array();
            foreach ($xpath->query('//input') as $fileLinkEl) {
                $attributeType = $fileLinkEl->getAttribute('type');
                if ($attributeType === 'text' || $attributeType === 'hidden' || $attributeType === 'checkbox' || $attributeType === 'radio') {
                    $fieldNames[str_replace(array('[', ']'), '', $fileLinkEl->getAttribute('name'))] = str_replace(array('[', ']'), '', $fileLinkEl->getAttribute('name'));
                }
            }
            foreach ($xpath->query('//select') as $fileLinkEl) {
                $fieldNames[$fileLinkEl->getAttribute('name')] = $fileLinkEl->getAttribute('name');
            }
            foreach ($xpath->query('//textarea') as $fileLinkEl) {
                $fieldNames[$fileLinkEl->getAttribute('name')] = $fileLinkEl->getAttribute('name');
            }

            return $fieldNames;
        }

        return array();
    }

    /**
     * parse form field names
     *
     * @param string $formName form name
     * @return array
     */
    public static function getExternalFormFields($formName)
    {
        $formGeneralConfigMapper = Leads_Mapper_LeadsBuilderFormGeneralConfigMapper::getInstance();
        $formConfigModel = $formGeneralConfigMapper->findByFormNameAndType($formName, Leads_Model_LeadsFormRulesGeneralConfigModel::FORM_TYPE_EXTERNAL);
        if ($formConfigModel instanceof Leads_Model_LeadsBuilderFormGeneralConfigModel) {
            $fieldTypes = array(
                Tools_LeadFormBuilderTools::FORM_FIELD_TYPE_INPUT_TEXT,
                Tools_LeadFormBuilderTools::FORM_FIELD_TYPE_PREDEFINED_EMAIL,
                Tools_LeadFormBuilderTools::FORM_FIELD_TYPE_DROPDOWN,
                Tools_LeadFormBuilderTools::FORM_FIELD_TYPE_PREDEFINED_NAME,
                Tools_LeadFormBuilderTools::FORM_FIELD_TYPE_GROUP,
                Tools_LeadFormBuilderTools::FORM_FIELD_TYPE_CHECKBOX,
                Tools_LeadFormBuilderTools::FORM_FIELD_TYPE_RADIO,
                Tools_LeadFormBuilderTools::FORM_FIELD_TYPE_TEXTAREA
            );
            $formFields = Leads_Mapper_LeadsBuilderFormFieldsMapper::getInstance()->getFieldsDataNames($formConfigModel->getId(), $fieldTypes);
        }

        if (empty($formFields)) {
            $formFields = array();
        }

        return $formFields;

    }

    /**
     * Analyze assigned form rules
     *
     * @param string $formName form name
     * @param string $formType form type (internal, external)
     * @param array $formData form data
     * @param Leads_Model_LeadsModel $leadModel lead Model
     * @param string $leadModelStatus status of the lead (new, old)
     * @return array
     */
    public static function analyzeFormRules(
        $formName,
        $formType,
        $formData,
        Leads_Model_LeadsModel $leadModel,
        $leadModelStatus = self::LEAD_MODEL_STATUS_OLD
    ) {
        $formRules = Leads_Mapper_LeadsFormRulesGeneralConfigMapper::getInstance()->getFormRules($formName, $formType);
        if (!empty($formRules)) {
            $ruleIds = self::getRuleIdsBasedOnData($formRules, $formData);
            if (!empty($ruleIds)) {
                self::applyRuleActions($ruleIds, $leadModel, $leadModelStatus, self::LEAD_RULES_TYPE_FORM, $formData);
            }
        }

        return array();
    }

    /**
     * Process lead or organization custom attributes
     *
     * @param array $formParams import data
     * @param int $id lead or organization id
     * @param string $type lead or organization type
     * @return array lead custom params
     */
    public static function processFormCustomParams($formParams, $id, $type = self::LEAD_TYPE_CUSTOM_FIELD)
    {
        $customParamTypePrefix = '_lead_c';
        if ($type === self::ORGANIZATION_TYPE_CUSTOM_FIELD) {
            $customParamTypePrefix = '_organization_c';
        }
        $customParamsExists = preg_grep('~'.$customParamTypePrefix.'.*~', array_keys($formParams));
        $attributes = array();
        $attributeData = array();
        if (!empty($customParamsExists)) {
            foreach ($customParamsExists as $customParamName) {
                $customParamValue = filter_var($formParams[$customParamName], FILTER_SANITIZE_STRING);
                $customAttributeName = str_replace($customParamTypePrefix, '',
                    strtolower(filter_var($customParamName, FILTER_SANITIZE_STRING)));
                if ($customParamValue !== '') {
                    $attributes[$customAttributeName] = $customParamValue;
                }
                $attributeData[$customParamName] = $customParamValue;
            }
            if (!empty($attributes)) {
                $customParamsWithNotMatchedData = self::saveCustomParams($id, $type, $attributes);
                if (!empty($customParamsWithNotMatchedData)) {
                    foreach ($customParamsWithNotMatchedData as $removeAttr) {
                        unset($attributeData[$removeAttr.$customParamTypePrefix]);
                    }
                }
            }
        }

        return $attributeData;
    }

    /**
     * Save custom fields data
     *
     * @param int $id lead or organization id
     * @param array $customParams custom params
     * @param string $type organization or lead
     * @return array
     */
    public static function saveCustomParams($id, $type, $customParams)
    {
        if (!empty($customParams)) {
            $customParamsWithNotMatchedData = array();
            if ($type === self::LEAD_TYPE_CUSTOM_FIELD) {
                $customParamConfigMapper = Leads_Mapper_LeadsCustomParamsConfigMapper::getInstance();
                $customParamsOptionsMapper = Leads_Mapper_LeadsCustomParamsOptionsDataMapper::getInstance();
            }
            if ($type === self::ORGANIZATION_TYPE_CUSTOM_FIELD) {
                $customParamConfigMapper = Leads_Mapper_LeadsOrganizationCustomParamsConfigMapper::getInstance();
                $customParamsOptionsMapper = Leads_Mapper_LeadsOrganizationCustomParamsOptionsDataMapper::getInstance();
            }

            foreach ($customParams as $fieldName => $fieldVal) {
                $customAttributeConfigModel = $customParamConfigMapper->getByName($fieldName);
                if (!$customAttributeConfigModel instanceof Leads_LeadsCustomParamsConfigModelInterface) {
                    continue;
                }

                $optionIds = array();

                $customAttributeConfigId = $customAttributeConfigModel->getId();
                $paramType = $customAttributeConfigModel->getParamType();
                $fieldVal = filter_var($fieldVal, FILTER_SANITIZE_STRING);
                if (in_array($paramType, self::$_customAttributesTypesWithPredefinedValues, true)) {
                    $predefinedValues = $customParamsOptionsMapper->findByCustomParamId($customAttributeConfigId, true);
                    if (empty($predefinedValues)) {
                        $customParamsWithNotMatchedData[$fieldName] = $fieldName;
                        continue;
                    }

                    $predefinedValues = array_map('mb_strtolower', $predefinedValues);
                    if ($paramType === Tools_LeadImportTools::ATTRIBUTE_TYPE_SELECT) {
                        if (!is_array($fieldVal)) {
                            $fieldValArray = explode(',', $fieldVal);
                        }
                        $fieldValArray = array_map("mb_strtolower", $fieldValArray);
                        if (is_array($fieldValArray)) {

                            $comparisonResult = array_intersect($predefinedValues, $fieldValArray);
                            if (count($comparisonResult) == 0) {
                                $customParamsWithNotMatchedData[$fieldName] = $fieldName;
                                continue;
                            }
                            $optionIds = $comparisonResult;

                        } else {
                            $optionId = array_search(mb_strtolower($fieldVal), $predefinedValues);
                            if ($optionId === false) {
                                $customParamsWithNotMatchedData[$fieldName] = $fieldName;
                                continue;
                            } else {
                                $optionIds[$optionId] = $fieldVal;
                            }
                        }

                    }

                    if ($paramType === Tools_LeadImportTools::ATTRIBUTE_TYPE_CHECKBOX) {
                        $optionId = array_search(mb_strtolower($fieldVal), $predefinedValues);
                        if ($optionId === false) {
                            $customParamsWithNotMatchedData[$fieldName] = $fieldName;
                            continue;
                        } else {
                            $optionIds[$optionId] = $fieldVal;
                        }
                    }
                }

                if ($paramType === Tools_LeadImportTools::ATTRIBUTE_TYPE_DATE) {
                    $defaultTimezone = 'UTC';
                    $currentTime = Tools_System_Tools::convertDateFromTimezone('now', $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'H:i:s');
                    $fieldVal = Tools_System_Tools::convertDateFromTimezone($fieldVal, $defaultTimezone, Tools_System_Tools::getUserTimezone(), 'Y-m-d ') . $currentTime;
                }

                if (!empty($optionIds) && is_array($optionIds)) {
                    foreach ($optionIds as $optionId => $fieldValue) {
                        self::saveCustomAttributesData($type, $id, $customAttributeConfigId, $paramType, $fieldVal, $optionId);
                    }
                } else {
                    self::saveCustomAttributesData($type, $id, $customAttributeConfigId, $paramType, $fieldVal);
                }

            }

            return $customParamsWithNotMatchedData;
        }
    }

    /**
     * Save custom attributes
     *
     * @param int $id lead or organization id
     * @param string $type organization or lead
     * @param int $customAttributeConfigId custom attribute id
     * @param string $paramType param type
     * @param string $fieldVal field value
     * @param string
     */
    public static function saveCustomAttributesData($type, $id, $customAttributeConfigId, $paramType, $fieldVal, $optionId = 0)
    {

        if ($type === self::LEAD_TYPE_CUSTOM_FIELD) {
            $customParamsMapper = Leads_Mapper_LeadsCustomParamsMapper::getInstance();
        }
        if ($type === self::ORGANIZATION_TYPE_CUSTOM_FIELD) {
            $customParamsMapper = Leads_Mapper_LeadsOrganizationCustomParamsMapper::getInstance();
        }

        $customAttributeModel = $customParamsMapper->getParamByIdAndParamId($id, $customAttributeConfigId);

        if (!$customAttributeModel instanceof Leads_LeadsCustomParamsModelInterface) {
            if ($type === self::LEAD_TYPE_CUSTOM_FIELD) {
                $customAttributeModel = new Leads_Model_LeadsCustomParamsModel();
                $customAttributeModel->setLeadId($id);
            }
            if ($type === self::ORGANIZATION_TYPE_CUSTOM_FIELD) {
                $customAttributeModel = new Leads_Model_LeadsOrganizationCustomParamsModel();
                $customAttributeModel->setOrganizationId($id);
            }
            $customAttributeModel->setParamId($customAttributeConfigId);
        }

        $currentParamValue = $customAttributeModel->getParamValue();
        $currentOptionId = $customAttributeModel->getParamsOptionId();
        if (empty($currentParamValue) || (empty($currentOptionId) && in_array($paramType, self::$_customAttributesTypesWithPredefinedValues, true))) {
            $customAttributeModel->setParamValue($fieldVal);
            if (in_array($paramType, self::$_customAttributesTypesWithPredefinedValues, true) && !empty($optionId)) {
                $customAttributeModel->setParamsOptionId($optionId);
            }
            $customParamsMapper->save($customAttributeModel);
        }
    }

}