<?php

/**
 * Class Lead_LeadGeolocationTools
 */
class Tools_LeadGeolocationTools
{

    /**
     * Prepare address for google map
     *
     * @param array $data
     * @return string
     */
    public static function prepareAddress(array $data)
    {
        $addressFields = array(
            'address',
            'city',
            'state',
            'country',
            'zip'
        );

        $address = '';
        foreach ($addressFields as $key) {
            $address .= !empty($data[$key]) ? $data[$key] . ' ' : '';
        }

        return $address;
    }

    /**
     * Get coordinates latitude and longitude
     *
     * @param array $addressFields 'address','city','state', 'country', 'zip'
     * @param string $apiKey
     * @return array
     */
    public static function getMapCoordinates(array $addressFields, $apiKey = '')
    {
        if (empty($apiKey)) {
            $configMapper = Application_Model_Mappers_ConfigMapper::getInstance();
            $configParams = $configMapper->getConfig();
            if (!empty($configParams['crmGoogleMapKey'])) {
                $apiKey = $configParams['crmGoogleMapKey'];
            }
        }

        if (empty($apiKey)) {
            return array('error' => '1', 'message' => 'api key not found');
        }

        $address = self::prepareAddress($addressFields);

        if (empty($address)) {
            return array();
        }

        $url = 'https://maps.google.com/maps/api/geocode/json?sensor=false&key=' . $apiKey . '&address=' . str_replace(' ', '+', $address);

        $response = file_get_contents($url);
        //generate array object from the response from the web
        $json = json_decode($response, true);
        if (!empty($json['error_message'])) {
            return array('error' => '1');
        }
        if (empty($json['results'])) {
            return array('lat' => '', 'lng' => '');
        }

        return array(
            'lat' => $json['results'][0]['geometry']['location']['lat'],
            'lng' => $json['results'][0]['geometry']['location']['lng']
        );
    }

    /**
     * Generate address hash
     * @param array $data 'address','city','state', 'country', 'zip'
     * @return string
     */
    public static function generateAddressHash(array $data)
    {
        $addressFields = array(
            'address',
            'city',
            'state',
            'country',
            'zip'
        );

        $address = '';
        foreach ($addressFields as $key) {
            $address .= !empty($data[$key]) ? $data[$key] . ' ' : '';
        }

        return sha1($address);
    }

    /**
     * Get address coord and save them to the coord log
     * @param array $addressFields 'address','city','state', 'country', 'zip'
     * @return array
     */
    public static function getGeolocationCoordinates(array $addressFields)
    {
        $addressHash = self::generateAddressHash($addressFields);
        $leadsHashedAddressCoordinatesMapper = Leads_Mapper_LeadsHashedAddressCoordinatesMapper::getInstance();
        $leadsHashedAddressCoordinatesModel = $leadsHashedAddressCoordinatesMapper->findByHash($addressHash);
        if ($leadsHashedAddressCoordinatesModel instanceof Leads_Model_LeadsHashedAddressCoordinatesModel) {
            return array('lat' => $leadsHashedAddressCoordinatesModel->getLat(), 'lng' => $leadsHashedAddressCoordinatesModel->getLng(), 'hash' => $leadsHashedAddressCoordinatesModel->getAddressHash());
        }

        $addressCoordinates = self::getMapCoordinates($addressFields);
        if (!empty($addressCoordinates['error'])) {
            return $addressCoordinates;
        }

        $leadsHashedAddressCoordinatesModel = new Leads_Model_LeadsHashedAddressCoordinatesModel();
        $leadsHashedAddressCoordinatesModel->setLat($addressCoordinates['lat']);
        $leadsHashedAddressCoordinatesModel->setLng($addressCoordinates['lng']);
        $leadsHashedAddressCoordinatesModel->setAddressHash($addressHash);
        $leadsHashedAddressCoordinatesMapper->save($leadsHashedAddressCoordinatesModel);
        $addressCoordinates['hash'] = $addressHash;

        return $addressCoordinates;
    }

    public static function generateAddressHashFromLeadModel(Leads_Model_LeadsModel $leadsModel)
    {
        $addressFields = array(
            'address' => $leadsModel->getLeadAddress(),
            'city' => $leadsModel->getLeadCity(),
            'state' => $leadsModel->getLeadStateCode(),
            'country' => $leadsModel->getLeadCountryCode(),
            'zip' => $leadsModel->getLeadZip(),
        );

        if (empty($addressFields['state'])) {
            $addressFields['state'] = '';
        }

        $addressCoordinates = self::getGeolocationCoordinates($addressFields);

        return $addressCoordinates;
    }

    /**
     * generate address coordinates and hash and attach it to lead model
     *
     * @param Leads_Model_LeadsModel $leadsModel
     * @return Leads_Model_LeadsModel
     */
    public static function generateAddressHashFromLeadModelAndAssignData(Leads_Model_LeadsModel $leadsModel)
    {
        $addressCoordinates = self::generateAddressHashFromLeadModel($leadsModel);
        if (!empty($addressCoordinates['error'])) {
            return $leadsModel;
        }

        $leadsModel->setLat($addressCoordinates['lat']);
        $leadsModel->setLng($addressCoordinates['lng']);
        $leadsModel->setAddressHash($addressCoordinates['hash']);
        $leadsModel->setLastCoordinatesProcessedDate(Tools_System_Tools::convertDateFromTimezone('now'));
        return $leadsModel;

    }

    /**
     * @param Leads_Model_LeadsOrganizationModel $organizationModel
     * @return array|string[]
     */
    public static function generateAddressHashFromOrganizationModel(Leads_Model_LeadsOrganizationModel $organizationModel)
    {
        $addressFields = array(
            'address' => $organizationModel->getOrganizationAddress(),
            'city' => $organizationModel->getOrganizationCity(),
            'state' => $organizationModel->getOrganizationStateCode(),
            'country' => $organizationModel->getOrganizationCountryCode(),
            'zip' => $organizationModel->getOrganizationZip(),
        );

        if (empty($addressFields['state'])) {
            $addressFields['state'] = '';
        }

        $addressCoordinates = self::getGeolocationCoordinates($addressFields);

        return $addressCoordinates;
    }

    /**
     * generate address coordinates and hash and attach it to organization model
     *
     * @param Leads_Model_LeadsOrganizationModel $organizationModel
     * @return Leads_Model_LeadsOrganizationModel
     */
    public static function generateAddressHashFromOrganizationModelAndAssignData(Leads_Model_LeadsOrganizationModel $organizationModel)
    {
        $addressCoordinates = self::generateAddressHashFromOrganizationModel($organizationModel);
        if (!empty($addressCoordinates['error'])) {
            return $organizationModel;
        }

        $organizationModel->setLat($addressCoordinates['lat']);
        $organizationModel->setLng($addressCoordinates['lng']);
        $organizationModel->setAddressHash($addressCoordinates['hash']);
        $organizationModel->setLastCoordinatesProcessedDate(Tools_System_Tools::convertDateFromTimezone('now'));
        return $organizationModel;

    }

    /**
     * @param string $lat
     * @param string $lng
     * @param string $radius radius in km
     * @return array
     */
    public static function prepareCoordinatesRange($lat, $lng, $radius)
    {
        $radiusDiffValue = $radius / 111;
        $radiusDiffValue = number_format($radiusDiffValue, 7, '.', '');
        $latitudeStart = $lat - $radiusDiffValue;
        $latitudeEnd = $lat + $radiusDiffValue;
        $longitudeStart = $lng - $radiusDiffValue;
        $longitudeEnd = $lng + $radiusDiffValue;

        $result = array(
            'latitudeStart' => $latitudeStart,
            'latitudeEnd' => $latitudeEnd,
            'longitudeStart' => $longitudeStart,
            'longitudeEnd' => $longitudeEnd
        );

        return $result;
    }

}
