<?php

require_once(__DIR__ . '/../../library/libphonenumber/vendor/autoload.php');

/**
 *
 * A PHP library for parsing, formatting, storing and validating international phone numbers.
 * This library is based on Google's libphonenumber.
 *
 * @link https://github.com/giggsey/libphonenumber-for-php
 *
 * Class Lead_LeadPhoneLibTools
 */
class Tools_LeadPhoneLibTools
{
    const PHONE_FORMAT_E164 = 0;

    const PHONE_FORMAT_INTERNATIONAL = 1;

    const PHONE_FORMAT_NATIONAL = 2;

    const PHONE_FORMAT_RFC3966 = 3;

    /**
     * Phone formats
     *
     * @var array
     */
    public static $_phoneFormatMapping = array(
        self::PHONE_FORMAT_E164,
        self::PHONE_FORMAT_INTERNATIONAL,
        self::PHONE_FORMAT_NATIONAL,
        self::PHONE_FORMAT_RFC3966
    );

    /**
     * Normalize phone number
     *
     * @param string $phoneNumber raw phone number
     * @param string $countryCode country code ISO 3166-1 alpha-2
     * @param int $format formatting method
     * INTERNATIONAL and NATIONAL formats are consistent with the definition in ITU-T Recommendation
     * E123. For example, the number of the Google Switzerland office will be written as
     * "+41 44 668 1800" in INTERNATIONAL format, and as "044 668 1800" in NATIONAL format.
     * E164 format is as per INTERNATIONAL format but with no formatting applied, e.g.
     * "+41446681800". RFC3966 is as per INTERNATIONAL format, but with all spaces and other
     * separating symbols replaced with a hyphen, and with any phone number extension appended with
     * ";ext=". It also will have a prefix of "tel:" added, e.g. "tel:+41-44-668-1800".
     *
     * Ex: E164, INTERNATIONAL, NATIONAL, RFC3966
     *
     * @param bool $forceRemoveCountryCode remove country code
     *
     * @return string
     */
    public static function normalizePhoneNumber($phoneNumber, $countryCode, $format, $forceRemoveCountryCode = false)
    {
        $phoneUtil = \libphonenumber\PhoneNumberUtil::getInstance();
        if (!in_array($format, self::$_phoneFormatMapping, true)) {
            return '';
        }

        try {
            $phoneNumberProto = $phoneUtil->parse($phoneNumber, $countryCode);
            $phoneNumber = $phoneUtil->format($phoneNumberProto, $format);
            if ($forceRemoveCountryCode === true) {
                $phoneCountryCodeValue = Tools_LeadTools::getCountryCodeValue($countryCode);
                $phoneNumber = str_replace($phoneCountryCodeValue, '', $phoneNumber);

                return $phoneNumber;
            }

            return $phoneNumber;
        } catch (\libphonenumber\NumberParseException $e) {
            return '';
        }
    }


    public static function getPhonesWithAndWithoutLeadingZero($phoneNumber, $countryCode = '')
    {

        $withAndWithoutLeadingNumber = array();
        $phoneUtil = \libphonenumber\PhoneNumberUtil::getInstance();
        try {
            if (empty($countryCode)) {
                $numberProto = $phoneUtil->parse($phoneNumber);
                $nationalNumber = $numberProto->getNationalNumber();
                $validatedCountryCode = '+' . $numberProto->getCountryCode();
                $withAndWithoutLeadingNumber['phoneWithZero'] = $validatedCountryCode . '0' . $nationalNumber;
                $withAndWithoutLeadingNumber['phoneWithoutZero'] = $validatedCountryCode . $nationalNumber;

            } else {
                $numberProto = $phoneUtil->parse($phoneNumber, $countryCode);
                $nationalNumber = $numberProto->getNationalNumber();
                $validatedCountryCode = '+' . $numberProto->getCountryCode();
                $withAndWithoutLeadingNumber['phoneWithZero'] = $validatedCountryCode . '0' . $nationalNumber;
                $withAndWithoutLeadingNumber['phoneWithoutZero'] = $validatedCountryCode . $nationalNumber;

            }

        } catch (\libphonenumber\NumberParseException $e) {
            return $withAndWithoutLeadingNumber;
        }

        return $withAndWithoutLeadingNumber;
    }

    /**
     * Parse raw mobile/phone number
     *
     * @param string $number phone|mobile number
     * @param int $format
     * @return array|\libphonenumber\PhoneNumber|string
     */
    public static function parseRawNumber($number, $format = self::PHONE_FORMAT_INTERNATIONAL)
    {
        $phoneUtil = \libphonenumber\PhoneNumberUtil::getInstance();
        if (!in_array($format, self::$_phoneFormatMapping, true)) {
            return '';
        }
        try {
            $info = $phoneUtil->parse($number);

            if ($info->getCountryCode() == '1') {
                $countryCode = 'US';
            } elseif($info->getCountryCode() == '44') {
                $countryCode = 'GB';
            } elseif($info->getCountryCode() == '61') {
                $countryCode = 'AU';
            } elseif($info->getCountryCode() == '47') {
                $countryCode = 'NO';
            } elseif($info->getCountryCode() == '7') {
                $countryCode = 'RU';
            } else {
                $allCountryCodes = array_flip(Zend_Locale::getTranslationList('phoneToTerritory'));
                $countryCode = $allCountryCodes[$info->getCountryCode()];
            }
            return array('number' => $info->getNationalNumber(), 'countryCode' => $countryCode, 'countryCodeValue' => $info->getCountryCode());
        } catch (\libphonenumber\NumberParseException $e) {
            return array();
        }
    }

}